@@ -19,9 +19,12 @@ construct command line strings by setting the values of each parameter.
 The finished command line strings are then normally invoked via the built-in
 Python module subprocess.
 """
+from __future__ import print_function
+from Bio._py3k import basestring
+
 import os
+import platform
 import sys
-import StringIO
 import subprocess
 import re
 
@@ -29,22 +32,26 @@ from subprocess import CalledProcessError as _ProcessCalledError
 
 from Bio import File
 
-#Use this regular expression to test the property names are going to
-#be valid as Python properties or arguments
-_re_prop_name = re.compile(r"[a-zA-Z][a-zA-Z0-9_]*")
+__docformat__ = "restructuredtext en"
+
+# Use this regular expression to test the property names are going to
+# be valid as Python properties or arguments
+_re_prop_name = re.compile(r"^[a-zA-Z][a-zA-Z0-9_]*$")
 assert _re_prop_name.match("t")
 assert _re_prop_name.match("test")
-assert _re_prop_name.match("_test") is None # we don't want private names
+assert _re_prop_name.match("_test") is None  # we don't want private names
 assert _re_prop_name.match("-test") is None
+assert _re_prop_name.match("any-hyphen") is None
+assert _re_prop_name.match("underscore_ok")
 assert _re_prop_name.match("test_name")
 assert _re_prop_name.match("test2")
-#These are reserved names in Python itself,
+# These are reserved names in Python itself,
 _reserved_names = ["and", "del", "from", "not", "while", "as", "elif",
                    "global", "or", "with", "assert", "else", "if", "pass",
                    "yield", "break", "except", "import", "print", "class",
                    "exec", "in", "raise", "continue", "finally", "is",
                    "return", "def", "for", "lambda", "try"]
-#These are reserved names due to the way the wrappers work
+# These are reserved names due to the way the wrappers work
 _local_reserved_names = ["set_parameter"]
 
 
@@ -60,8 +67,8 @@ class ApplicationError(_ProcessCalledError):
     >>> err = ApplicationError(-11, "helloworld", "", "Some error text")
     >>> err.returncode, err.cmd, err.stdout, err.stderr
     (-11, 'helloworld', '', 'Some error text')
-    >>> print err
-    Command 'helloworld' returned non-zero exit status -11, 'Some error text'
+    >>> print(err)
+    Non-zero return code -11 from 'helloworld', message 'Some error text'
 
     """
     def __init__(self, returncode, cmd, stdout="", stderr=""):
@@ -71,17 +78,17 @@ class ApplicationError(_ProcessCalledError):
         self.stderr = stderr
 
     def __str__(self):
-        #get first line of any stderr message
+        # get first line of any stderr message
         try:
             msg = self.stderr.lstrip().split("\n", 1)[0].rstrip()
         except:
             msg = ""
         if msg:
-            return "Command '%s' returned non-zero exit status %d, %r" \
-                   % (self.cmd, self.returncode, msg)
+            return "Non-zero return code %d from %r, message %r" \
+                   % (self.returncode, self.cmd, msg)
         else:
-            return "Command '%s' returned non-zero exit status %d" \
-                   % (self.cmd, self.returncode)
+            return "Non-zero return code %d from %r" \
+                   % (self.returncode, self.cmd)
 
     def __repr__(self):
         return "ApplicationError(%i, %s, %s, %s)" \
@@ -135,7 +142,7 @@ class AbstractCommandline(object):
     >>> water_cmd.asequence = "asis:ACCCGGGCGCGGT"
     >>> water_cmd.bsequence = "asis:ACCCGAGCGCGGT"
     >>> water_cmd.outfile = "temp_water.txt"
-    >>> print water_cmd
+    >>> print(water_cmd)
     water -outfile=temp_water.txt -asequence=asis:ACCCGGGCGCGGT -bsequence=asis:ACCCGAGCGCGGT -gapopen=10 -gapextend=0.5
     >>> water_cmd
     WaterCommandline(cmd='water', outfile='temp_water.txt', asequence='asis:ACCCGGGCGCGGT', bsequence='asis:ACCCGAGCGCGGT', gapopen=10, gapextend=0.5)
@@ -157,18 +164,18 @@ class AbstractCommandline(object):
     ...                              asequence="asis:ACCCGGGCGCGGT",
     ...                              bsequence="asis:ACCCGAGCGCGGT",
     ...                              outfile="temp_water.txt")
-    >>> print water_cmd
+    >>> print(water_cmd)
     "C:\Program Files\EMBOSS\water.exe" -outfile=temp_water.txt -asequence=asis:ACCCGGGCGCGGT -bsequence=asis:ACCCGAGCGCGGT -gapopen=10 -gapextend=0.5
 
     Notice that since the path name includes a space it has automatically
     been quoted.
 
     """
-    #TODO - Replace the above example since EMBOSS doesn't work properly
-    #if installed into a folder with a space like "C:\Program Files\EMBOSS"
+    # TODO - Replace the above example since EMBOSS doesn't work properly
+    # if installed into a folder with a space like "C:\Program Files\EMBOSS"
 
-    #Note the call example above is not a doctest as we can't handle EMBOSS
-    #(or any other tool) being missing in the unit tests.
+    # Note the call example above is not a doctest as we can't handle EMBOSS
+    # (or any other tool) being missing in the unit tests.
     def __init__(self, cmd, **kwargs):
         """Create a new instance of a command line wrapper object."""
         # Init method - should be subclassed!
@@ -192,7 +199,7 @@ class AbstractCommandline(object):
             parameters = self.parameters
         except AttributeError:
             raise AttributeError("Subclass should have defined self.parameters")
-        #Create properties for each parameter at run time
+        # Create properties for each parameter at run time
         aliases = set()
         for p in parameters:
             if not p.names:
@@ -218,7 +225,7 @@ class AbstractCommandline(object):
                                  "way the AbstractCommandline class works"
                                  % repr(name))
 
-            #Beware of binding-versus-assignment confusion issues
+            # Beware of binding-versus-assignment confusion issues
             def getter(name):
                 return lambda x: x._get_parameter(name)
 
@@ -238,7 +245,7 @@ class AbstractCommandline(object):
                        "argument value required." % p.names[0]
             prop = property(getter(name), setter(name), deleter(name), doc)
             setattr(self.__class__, name, prop)  # magic!
-        for key, value in kwargs.iteritems():
+        for key, value in kwargs.items():
             self.set_parameter(key, value)
 
     def _validate(self):
@@ -250,11 +257,11 @@ class AbstractCommandline(object):
         override it.
         """
         for p in self.parameters:
-            #Check for missing required parameters:
+            # Check for missing required parameters:
             if p.is_required and not(p.is_set):
                 raise ValueError("Parameter %s is not set."
                                  % p.names[-1])
-            #Also repeat the parameter validation here, just in case?
+            # Also repeat the parameter validation here, just in case?
 
     def __str__(self):
         """Make the commandline string with the currently set options.
@@ -265,7 +272,7 @@ class AbstractCommandline(object):
         >>> cline.asequence = "asis:ACCCGGGCGCGGT"
         >>> cline.bsequence = "asis:ACCCGAGCGCGGT"
         >>> cline.outfile = "temp_water.txt"
-        >>> print cline
+        >>> print(cline)
         water -outfile=temp_water.txt -asequence=asis:ACCCGGGCGCGGT -bsequence=asis:ACCCGAGCGCGGT -gapopen=10 -gapextend=0.5
         >>> str(cline)
         'water -outfile=temp_water.txt -asequence=asis:ACCCGGGCGCGGT -bsequence=asis:ACCCGAGCGCGGT -gapopen=10 -gapextend=0.5'
@@ -274,9 +281,9 @@ class AbstractCommandline(object):
         commandline = "%s " % _escape_filename(self.program_name)
         for parameter in self.parameters:
             if parameter.is_set:
-                #This will include a trailing space:
+                # This will include a trailing space:
                 commandline += str(parameter)
-        return commandline.strip() # remove trailing space
+        return commandline.strip()  # remove trailing space
 
     def __repr__(self):
         """Return a representation of the command line object for debugging.
@@ -287,7 +294,7 @@ class AbstractCommandline(object):
         >>> cline.asequence = "asis:ACCCGGGCGCGGT"
         >>> cline.bsequence = "asis:ACCCGAGCGCGGT"
         >>> cline.outfile = "temp_water.txt"
-        >>> print cline
+        >>> print(cline)
         water -outfile=temp_water.txt -asequence=asis:ACCCGGGCGCGGT -bsequence=asis:ACCCGAGCGCGGT -gapopen=10 -gapextend=0.5
         >>> cline
         WaterCommandline(cmd='water', outfile='temp_water.txt', asequence='asis:ACCCGGGCGCGGT', bsequence='asis:ACCCGAGCGCGGT', gapopen=10, gapextend=0.5)
@@ -324,7 +331,7 @@ class AbstractCommandline(object):
         if not cleared_option:
             raise ValueError("Option name %s was not found." % name)
 
-    def set_parameter(self, name, value = None):
+    def set_parameter(self, name, value=None):
         """Set a commandline option for a program (OBSOLETE).
 
         Every parameter is available via a property and as a named
@@ -387,7 +394,7 @@ class AbstractCommandline(object):
         Traceback (most recent call last):
         ...
         ValueError: Option name csequence was not found.
-        >>> print cline
+        >>> print(cline)
         water -stdout -asequence=a.fasta -bsequence=b.fasta -gapopen=10 -gapextend=0.5
 
         This workaround uses a whitelist of object attributes, and sets the
@@ -426,14 +433,14 @@ class AbstractCommandline(object):
         process' environment variables are used. See Python's subprocess
         module documentation for more details.
 
-        Default example usage:
+        Default example usage::
 
-        from Bio.Emboss.Applications import WaterCommandline
-        water_cmd = WaterCommandline(gapopen=10, gapextend=0.5,
-                                     stdout=True, auto=True,
-                                     asequence="a.fasta", bsequence="b.fasta")
-        print "About to run:\n%s" % water_cmd
-        std_output, err_output = water_cmd()
+            from Bio.Emboss.Applications import WaterCommandline
+            water_cmd = WaterCommandline(gapopen=10, gapextend=0.5,
+                                         stdout=True, auto=True,
+                                         asequence="a.fasta", bsequence="b.fasta")
+            print("About to run: %s" % water_cmd)
+            std_output, err_output = water_cmd()
 
         This functionality is similar to subprocess.check_output() added in
         Python 2.7. In general if you require more control over running the
@@ -455,25 +462,37 @@ class AbstractCommandline(object):
             stderr_arg = open(os.devnull, "w")
         elif isinstance(stderr, basestring):
             if stdout == stderr:
-                stderr_arg = stdout_arg #Write both to the same file
+                stderr_arg = stdout_arg  # Write both to the same file
             else:
                 stderr_arg = open(stderr, "w")
         else:
             stderr_arg = subprocess.PIPE
 
-        #We may not need to supply any piped input, but we setup the
-        #standard input pipe anyway as a work around for a python
-        #bug if this is called from a Windows GUI program.  For
-        #details, see http://bugs.python.org/issue1124861
+        # We may not need to supply any piped input, but we setup the
+        # standard input pipe anyway as a work around for a python
+        # bug if this is called from a Windows GUI program.  For
+        # details, see http://bugs.python.org/issue1124861
         #
-        #Using universal newlines is important on Python 3, this
-        #gives unicode handles rather than bytes handles.
+        # Using universal newlines is important on Python 3, this
+        # gives unicode handles rather than bytes handles.
+
+        # Windows 7 and 8 want shell = True
+        # platform is easier to understand that sys to determine
+        # windows version
+        if sys.platform != "win32":
+            use_shell = True
+        else:
+            win_ver = platform.win32_ver()[0]
+            if win_ver in ["7", "8"]:
+                use_shell = True
+            else:
+                use_shell = False
         child_process = subprocess.Popen(str(self), stdin=subprocess.PIPE,
                                          stdout=stdout_arg, stderr=stderr_arg,
                                          universal_newlines=True,
                                          cwd=cwd, env=env,
-                                         shell=(sys.platform!="win32"))
-        #Use .communicate as can get deadlocks with .wait(), see Bug 2804
+                                         shell=use_shell)
+        # Use .communicate as can get deadlocks with .wait(), see Bug 2804
         stdout_str, stderr_str = child_process.communicate(stdin)
         if not stdout:
             assert not stdout_str, stdout_str
@@ -481,12 +500,14 @@ class AbstractCommandline(object):
             assert not stderr_str, stderr_str
         return_code = child_process.returncode
 
-        #Particularly important to close handles on Jython and PyPy
-        #(where garbage collection is less predictable) and on Windows
-        #(where cannot delete files with an open handle):
-        if stdout and isinstance(stdout, basestring):
+        # Particularly important to close handles on Jython and PyPy
+        # (where garbage collection is less predictable) and on Windows
+        # (where cannot delete files with an open handle):
+        if not stdout or isinstance(stdout, basestring):
+            # We opened /dev/null or a file
             stdout_arg.close()
-        if stderr and isinstance(stderr, basestring) and stdout != stderr:
+        if not stderr or (isinstance(stderr, basestring) and stdout != stderr):
+            # We opened /dev/null or a file
             stderr_arg.close()
 
         if return_code:
@@ -523,7 +544,7 @@ class _Option(_AbstractParameter):
     (eg ["-a", "--append", "append"]). The first name in list is used
     when building the command line. The last name in the list is a
     "human readable" name describing the option in one word. This
-    must be a valid Python identifer as it is used as the property
+    must be a valid Python identifier as it is used as the property
     name and as a keyword argument, and should therefore follow PEP8
     naming.
 
@@ -596,7 +617,7 @@ class _Switch(_AbstractParameter):
     "human readable" name describing the option in one word. This
     must be a valid Python identifer as it is used as the property
     name and as a keyword argument, and should therefore follow PEP8
-    naming. 
+    naming.
 
     o description -- a description of the option. This is used as
     the property docstring.
@@ -633,7 +654,7 @@ class _Argument(_AbstractParameter):
     """
     def __init__(self, names, description, filename=False,
                  checker_function=None, is_required=False):
-        #if len(names) != 1:
+        # if len(names) != 1:
         #    raise ValueError("The names argument to _Argument should be a "
         #                     "single entry list with a PEP8 property name.")
         self.names = names
@@ -655,6 +676,24 @@ class _Argument(_AbstractParameter):
             return "%s " % self.value
 
 
+class _ArgumentList(_Argument):
+    """Represent a variable list of arguments on a command line, e.g. multiple filenames."""
+    # TODO - Option to require at least one value? e.g. min/max count?
+
+    def __str__(self):
+        assert isinstance(self.value, list), \
+                "Arguments should be a list"
+        assert self.value, "Requires at least one filename"
+        # A trailing space is required so that parameters following the last filename
+        # do not appear merged.
+        # e.g.:  samtools cat in1.bam in2.bam-o out.sam  [without trailing space][Incorrect]
+        #        samtools cat in1.bam in2.bam -o out.sam  [with trailing space][Correct]
+        if self.is_filename:
+            return " ".join(_escape_filename(v) for v in self.value) + " "
+        else:
+            return " ".join(self.value) + " "
+
+
 class _StaticArgument(_AbstractParameter):
     """Represent a static (read only) argument on a commandline.
 
@@ -676,17 +715,17 @@ def _escape_filename(filename):
 
     Note this will not add quotes if they are already included:
 
-    >>> print _escape_filename('example with spaces')
+    >>> print((_escape_filename('example with spaces')))
     "example with spaces"
-    >>> print _escape_filename('"example with spaces"')
+    >>> print((_escape_filename('"example with spaces"')))
     "example with spaces"
     """
-    #Is adding the following helpful
-    #if os.path.isfile(filename):
-    #    #On Windows, if the file exists, we can ask for
-    #    #its alternative short name (DOS style 8.3 format)
-    #    #which has no spaces in it.  Note that this name
-    #    #is not portable between machines, or even folder!
+    # Is adding the following helpful
+    # if os.path.isfile(filename):
+    #    # On Windows, if the file exists, we can ask for
+    #    # its alternative short name (DOS style 8.3 format)
+    #    # which has no spaces in it.  Note that this name
+    #    # is not portable between machines, or even folder!
     #    try:
     #        import win32api
     #        short = win32api.GetShortPathName(filename)
@@ -696,9 +735,9 @@ def _escape_filename(filename):
     #        pass
     if " " not in filename:
         return filename
-    #We'll just quote it - works on Windows, Mac OS X etc
+    # We'll just quote it - works on Windows, Mac OS X etc
     if filename.startswith('"') and filename.endswith('"'):
-        #Its already quoted
+        # Its already quoted
         return filename
     else:
         return '"%s"' % filename
@@ -710,5 +749,5 @@ def _test():
     doctest.testmod(verbose=1)
 
 if __name__ == "__main__":
-    #Run the doctests
+    # Run the doctests
     _test()
