@@ -4,8 +4,7 @@
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.
 
-"""
-This module provides code to work with the KEGG Ligand/Compound database.
+"""Code to work with the KEGG Ligand/Compound database.
 
 Functions:
 parse - Returns an iterator giving Record objects.
@@ -15,18 +14,18 @@ Record - A representation of a KEGG Ligand/Compound.
 """
 
 # other Biopython stuff
+from __future__ import print_function
+
 from Bio.KEGG import _write_kegg
 from Bio.KEGG import _wrap_kegg
 
 
 # Set up line wrapping rules (see Bio.KEGG._wrap_kegg)
 name_wrap = [0, "",
-             (" ","$",1,1),
-             ("-","$",1,1)]
-id_wrap = lambda indent : [indent, "",
-                           (" ","",1,0)]
-struct_wrap = lambda indent : [indent, "",
-                               ("  ","",1,1)]
+             (" ", "$", 1, 1),
+             ("-", "$", 1, 1)]
+id_wrap = lambda indent: [indent, "", (" ", "", 1, 0)]
+struct_wrap = lambda indent: [indent, "", ("  ", "", 1, 1)]
 
 
 class Record(object):
@@ -48,14 +47,14 @@ class Record(object):
 
         Create a new Record.
         """
-        self.entry      = ""
-        self.name       = []
-        self.formula    = ""
-        self.mass       = ""
-        self.pathway    = []
-        self.enzyme     = []
+        self.entry = ""
+        self.name = []
+        self.formula = ""
+        self.mass = ""
+        self.pathway = []
+        self.enzyme = []
         self.structures = []
-        self.dblinks    = []
+        self.dblinks = []
 
     def __str__(self):
         """__str__(self)
@@ -78,7 +77,7 @@ class Record(object):
 
     def _name(self):
         return _write_kegg("NAME",
-                           [_wrap_kegg(l, wrap_rule = name_wrap)
+                           [_wrap_kegg(l, wrap_rule=name_wrap)
                             for l in self.name])
 
     def _formula(self):
@@ -94,7 +93,7 @@ class Record(object):
         for entry in self.pathway:
             s.append(entry[0] + ": " + entry[1] + "  " + entry[2])
         return _write_kegg("PATHWAY",
-                           [_wrap_kegg(l, wrap_rule = id_wrap(16))
+                           [_wrap_kegg(l, wrap_rule=id_wrap(16))
                             for l in s])
 
     def _enzyme(self):
@@ -106,14 +105,14 @@ class Record(object):
                 t = entry[0]
             s = s + t.ljust(16)
         return _write_kegg("ENZYME",
-                            [_wrap_kegg(s, wrap_rule = id_wrap(0))])
+                            [_wrap_kegg(s, wrap_rule=id_wrap(0))])
 
     def _structures(self):
         s = []
         for entry in self.structures:
             s.append(entry[0] + ": " + "  ".join(entry[1]) + "  ")
         return _write_kegg("STRUCTURES",
-                           [_wrap_kegg(l, wrap_rule = struct_wrap(5))
+                           [_wrap_kegg(l, wrap_rule=struct_wrap(5))
                             for l in s])
 
     def _dblinks(self):
@@ -121,7 +120,7 @@ class Record(object):
         for entry in self.dblinks:
             s.append(entry[0] + ": " + " ".join(entry[1]))
         return _write_kegg("DBLINKS",
-                           [_wrap_kegg(l, wrap_rule = id_wrap(9))
+                           [_wrap_kegg(l, wrap_rule=id_wrap(9))
                             for l in s])
 
 
@@ -132,9 +131,9 @@ def parse(handle):
     example, using one of the example KEGG files in the Biopython
     test suite,
 
-    >>> handle = open("KEGG/compound.sample")
-    >>> for record in parse(handle):
-    ...     print record.entry, record.name[0]
+    >>> with open("KEGG/compound.sample") as handle:
+    ...     for record in parse(handle):
+    ...         print("%s %s" % (record.entry, record.name[0]))
     ...
     C00023 Iron
     C00017 Protein
@@ -144,7 +143,6 @@ def parse(handle):
     C00348 Undecaprenyl phosphate
     C00349 2-Methyl-3-oxopropanoate
     C01386 NH2Mec
-    >>> handle.close()
 
     """
     record = Record()
@@ -174,7 +172,7 @@ def parse(handle):
                 record.enzyme.append(enzyme)
         elif keyword=="PATHWAY     ":
             if data[:5]=='PATH:':
-                path, map, name = data.split(None,2)
+                path, map, name = data.split(None, 2)
                 pathway = (path[:-1], map, name)
                 record.pathway.append(pathway)
             else:
