@@ -12,7 +12,7 @@ Simple example with multiple chains,
     >>> structure = PDBParser().get_structure('2BEG', 'PDB/2BEG.pdb')
     >>> ppb=PPBuilder()
     >>> for pp in ppb.build_peptides(structure):
-    ...     print pp.get_sequence()
+    ...     print(pp.get_sequence())
     LVFFAEDVGSNKGAIIGLMVGGVVIA
     LVFFAEDVGSNKGAIIGLMVGGVVIA
     LVFFAEDVGSNKGAIIGLMVGGVVIA
@@ -27,7 +27,7 @@ in this case selenomethionine (MSE):
     >>> structure = PDBParser().get_structure('1A8O', 'PDB/1A8O.pdb')
     >>> ppb=PPBuilder()
     >>> for pp in ppb.build_peptides(structure):
-    ...     print pp.get_sequence()
+    ...     print(pp.get_sequence())
     DIRQGPKEPFRDYVDRFYKTLRAEQASQEVKNW
     TETLLVQNANPDCKTILKALGPGATLEE
     TACQG
@@ -35,10 +35,10 @@ in this case selenomethionine (MSE):
 If you want to, you can include non-standard amino acids in the peptides:
 
     >>> for pp in ppb.build_peptides(structure, aa_only=False):
-    ...     print pp.get_sequence()
-    ...     print pp.get_sequence()[0], pp[0].get_resname()
-    ...     print pp.get_sequence()[-7], pp[-7].get_resname()
-    ...     print pp.get_sequence()[-6], pp[-6].get_resname()
+    ...     print(pp.get_sequence())
+    ...     print("%s %s" % (pp.get_sequence()[0], pp[0].get_resname()))
+    ...     print("%s %s" % (pp.get_sequence()[-7], pp[-7].get_resname()))
+    ...     print("%s %s" % (pp.get_sequence()[-6], pp[-6].get_resname()))
     MDIRQGPKEPFRDYVDRFYKTLRAEQASQEVKNWMTETLLVQNANPDCKTILKALGPGATLEEMMTACQG
     M MSE
     M MSE
@@ -48,6 +48,9 @@ In this case the selenomethionines (the first and also seventh and sixth from
 last residues) have been shown as M (methionine) by the get_sequence method.
 """
 
+from __future__ import print_function
+from Bio._py3k import basestring
+
 import warnings
 
 from Bio.Alphabet import generic_protein
@@ -174,7 +177,7 @@ def is_aa(residue, standard=False):
     >>> is_aa('FME', standard=True)
     False
     """
-    #TODO - What about special cases like XXX, can they appear in PDB files?
+    # TODO - What about special cases like XXX, can they appear in PDB files?
     if not isinstance(residue, basestring):
         residue=residue.get_resname()
     residue=residue.upper()
@@ -315,9 +318,9 @@ class _PPBuilder:
         if is_aa(residue, standard=standard_aa_only):
             return True
         elif not standard_aa_only and "CA" in residue.child_dict:
-            #It has an alpha carbon...
-            #We probably need to update the hard coded list of
-            #non-standard residues, see function is_aa for details.
+            # It has an alpha carbon...
+            # We probably need to update the hard coded list of
+            # non-standard residues, see function is_aa for details.
             warnings.warn("Assuming residue %s is an unknown modified "
                           "amino acid" % residue.get_resname())
             return True
@@ -351,11 +354,11 @@ class _PPBuilder:
         for chain in chain_list:
             chain_it=iter(chain)
             try:
-                prev_res = chain_it.next()
+                prev_res = next(chain_it)
                 while not accept(prev_res, aa_only):
-                    prev_res = chain_it.next()
+                    prev_res = next(chain_it)
             except StopIteration:
-                #No interesting residues at all in this chain
+                # No interesting residues at all in this chain
                 continue
             pp=None
             for next_res in chain_it:
@@ -368,8 +371,8 @@ class _PPBuilder:
                         pp_list.append(pp)
                     pp.append(next_res)
                 else:
-                    #Either too far apart, or one of the residues is unwanted.
-                    #End the current peptide
+                    # Either too far apart, or one of the residues is unwanted.
+                    # End the current peptide
                     pp=None
                 prev_res=next_res
         return pp_list
@@ -460,24 +463,24 @@ if __name__=="__main__":
 
     ppb=PPBuilder()
 
-    print "C-N"
+    print("C-N")
     for pp in ppb.build_peptides(s):
-        print pp.get_sequence()
+        print(pp.get_sequence())
     for pp in ppb.build_peptides(s[0]):
-        print pp.get_sequence()
+        print(pp.get_sequence())
     for pp in ppb.build_peptides(s[0]["A"]):
-        print pp.get_sequence()
+        print(pp.get_sequence())
 
     for pp in ppb.build_peptides(s):
         for phi, psi in pp.get_phi_psi_list():
-            print phi, psi
+            print("%f %f" % (phi, psi))
 
     ppb=CaPPBuilder()
 
-    print "CA-CA"
+    print("CA-CA")
     for pp in ppb.build_peptides(s):
-        print pp.get_sequence()
+        print(pp.get_sequence())
     for pp in ppb.build_peptides(s[0]):
-        print pp.get_sequence()
+        print(pp.get_sequence())
     for pp in ppb.build_peptides(s[0]["A"]):
-        print pp.get_sequence()
+        print(pp.get_sequence())
