@@ -3,16 +3,12 @@
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.
 
-from Numeric import array, sum, sqrt, arccos, matrixmultiply, transpose, cos, \
-        sin, zeros, trace
-from math import acos
-from LinearAlgebra import determinant, eigenvectors
-from MLab import eye
-import sys
-from math import pi
+"""Vector class, including rotation-related functions."""
 
+from __future__ import print_function
+
+import numpy
 
-__doc__="Vector class, including rotation-related functions."
 
 def m2rotaxis(m):
     """
@@ -20,41 +16,41 @@ def m2rotaxis(m):
     """
     # Angle always between 0 and pi
     # Sense of rotation is defined by axis orientation
-    t=0.5*(trace(m)-1)
+    t=0.5*(numpy.trace(m)-1)
     t=max(-1, t)
     t=min(1, t)
-    angle=acos(t)
+    angle=numpy.arccos(t)
     if angle<1e-15:
         # Angle is 0
-        return 0.0, Vector(1,0,0)
-    elif angle<pi:
+        return 0.0, Vector(1, 0, 0)
+    elif angle<numpy.pi:
         # Angle is smaller than pi
-        x=m[2,1]-m[1,2]
-        y=m[0,2]-m[2,0]
-        z=m[1,0]-m[0,1]
-        axis=Vector(x,y,z)
+        x=m[2, 1]-m[1, 2]
+        y=m[0, 2]-m[2, 0]
+        z=m[1, 0]-m[0, 1]
+        axis=Vector(x, y, z)
         axis.normalize()
         return angle, axis
     else:
         # Angle is pi - special case!
-        m00=m[0,0]
-        m11=m[1,1]
-        m22=m[2,2]
+        m00=m[0, 0]
+        m11=m[1, 1]
+        m22=m[2, 2]
         if m00>m11 and m00>m22:
-            x=sqrt(m00-m11-m22+0.5)
-            y=m[0,1]/(2*x)
-            z=m[0,2]/(2*x)
+            x=numpy.sqrt(m00-m11-m22+0.5)
+            y=m[0, 1]/(2*x)
+            z=m[0, 2]/(2*x)
         elif m11>m00 and m11>m22:
-            y=sqrt(m11-m00-m22+0.5)
-            x=m[0,1]/(2*y)
-            z=m[1,2]/(2*y)
+            y=numpy.sqrt(m11-m00-m22+0.5)
+            x=m[0, 1]/(2*y)
+            z=m[1, 2]/(2*y)
         else:
-            z=sqrt(m22-m00-m11+0.5)
-            x=m[0,2]/(2*z)
-            y=m[1,2]/(2*z)
-        axis=Vector(x,y,z)
+            z=numpy.sqrt(m22-m00-m11+0.5)
+            x=m[0, 2]/(2*z)
+            y=m[1, 2]/(2*z)
+        axis=Vector(x, y, z)
         axis.normalize()
-        return pi, axis
+        return numpy.pi, axis
 
 
 def vector_to_axis(line, point):
@@ -72,7 +68,7 @@ def vector_to_axis(line, point):
     line=line.normalized()
     np=point.norm()
     angle=line.angle(point)
-    return point-line**(np*cos(angle))
+    return point-line**(np*numpy.cos(angle))
 
 
 def rotaxis2m(theta, vector):
@@ -80,9 +76,9 @@ def rotaxis2m(theta, vector):
     Calculate a left multiplying rotation matrix that rotates
     theta rad around vector.
 
-    Example: 
-    
-        >>> m=rotaxis(pi, Vector(1,0,0))
+    Example:
+
+        >>> m=rotaxis(pi, Vector(1, 0, 0))
         >>> rotated_vector=any_vector.left_multiply(m)
 
     @type theta: float
@@ -96,58 +92,62 @@ def rotaxis2m(theta, vector):
     """
     vector=vector.copy()
     vector.normalize()
-    c=cos(theta)
-    s=sin(theta)
+    c=numpy.cos(theta)
+    s=numpy.sin(theta)
     t=1-c
-    x,y,z=vector.get_array()
-    rot=zeros((3,3), "d")
+    x, y, z=vector.get_array()
+    rot=numpy.zeros((3, 3))
     # 1st row
-    rot[0,0]=t*x*x+c
-    rot[0,1]=t*x*y-s*z
-    rot[0,2]=t*x*z+s*y
+    rot[0, 0]=t*x*x+c
+    rot[0, 1]=t*x*y-s*z
+    rot[0, 2]=t*x*z+s*y
     # 2nd row
-    rot[1,0]=t*x*y+s*z
-    rot[1,1]=t*y*y+c
-    rot[1,2]=t*y*z-s*x
+    rot[1, 0]=t*x*y+s*z
+    rot[1, 1]=t*y*y+c
+    rot[1, 2]=t*y*z-s*x
     # 3rd row
-    rot[2,0]=t*x*z-s*y
-    rot[2,1]=t*y*z+s*x
-    rot[2,2]=t*z*z+c
+    rot[2, 0]=t*x*z-s*y
+    rot[2, 1]=t*y*z+s*x
+    rot[2, 2]=t*z*z+c
     return rot
 
 rotaxis=rotaxis2m
 
-def refmat(p,q):
+
+def refmat(p, q):
     """
     Return a (left multiplying) matrix that mirrors p onto q.
 
     Example:
-        >>> mirror=refmat(p,q)
+        >>> mirror=refmat(p, q)
         >>> qq=p.left_multiply(mirror)
-        >>> print q, qq # q and qq should be the same
+        >>> print(q)
+        >>> print(qq) # q and qq should be the same
 
     @type p,q: L{Vector}
-    @return: The mirror operation, a 3x3 Numeric array. 
+    @return: The mirror operation, a 3x3 Numeric array.
     """
     p.normalize()
     q.normalize()
     if (p-q).norm()<1e-5:
-        return eye(3)
+        return numpy.identity(3)
     pq=p-q
     pq.normalize()
     b=pq.get_array()
     b.shape=(3, 1)
-    i=eye(3)
-    ref=i-2*matrixmultiply(b, transpose(b))
+    i=numpy.identity(3)
+    ref=i-2*numpy.dot(b, numpy.transpose(b))
     return ref
 
-def rotmat(p,q):
+
+def rotmat(p, q):
     """
     Return a (left multiplying) matrix that rotates p onto q.
 
     Example:
-        >>> r=rotmat(p,q)
-        >>> print q, p.left_multiply(r)
+        >>> r=rotmat(p, q)
+        >>> print(q)
+        >>> print(p.left_multiply(r))
 
     @param p: moving vector
     @type p: L{Vector}
@@ -158,9 +158,10 @@ def rotmat(p,q):
     @return: rotation matrix that rotates p onto q
     @rtype: 3x3 Numeric array
     """
-    rot=matrixmultiply(refmat(q, -p), refmat(p, -p))
+    rot=numpy.dot(refmat(q, -p), refmat(p, -p))
     return rot
 
+
 def calc_angle(v1, v2, v3):
     """
     Calculate the angle between 3 vectors
@@ -168,7 +169,7 @@ def calc_angle(v1, v2, v3):
 
     @param v1, v2, v3: the tree points that define the angle
     @type v1, v2, v3: L{Vector}
-    
+
     @return: angle
     @rtype: float
     """
@@ -176,6 +177,7 @@ def calc_angle(v1, v2, v3):
     v3=v3-v2
     return v1.angle(v3)
 
+
 def calc_dihedral(v1, v2, v3, v4):
     """
     Calculate the dihedral angle between 4 vectors
@@ -201,22 +203,24 @@ def calc_dihedral(v1, v2, v3, v4):
         pass
     return angle
 
-class Vector:
+
+class Vector(object):
     "3D vector"
 
     def __init__(self, x, y=None, z=None):
         if y is None and z is None:
             # Array, list, tuple...
             if len(x)!=3:
-                raise "Vector: x is not a list/tuple/array of 3 numbers"
-            self._ar=array(x, 'd')
+                raise ValueError("Vector: x is not a "
+                                 "list/tuple/array of 3 numbers")
+            self._ar=numpy.array(x, 'd')
         else:
             # Three numbers
-            self._ar=array((x, y, z), 'd')
+            self._ar=numpy.array((x, y, z), 'd')
 
     def __repr__(self):
-        x,y,z=self._ar
-        return "<Vector %.2f, %.2f, %.2f>" % (x,y,z)
+        x, y, z=self._ar
+        return "<Vector %.2f, %.2f, %.2f>" % (x, y, z)
 
     def __neg__(self):
         "Return Vector(-x, -y, -z)"
@@ -228,7 +232,7 @@ class Vector:
         if isinstance(other, Vector):
             a=self._ar+other._ar
         else:
-            a=self._ar+array(other)
+            a=self._ar+numpy.array(other)
         return Vector(a)
 
     def __sub__(self, other):
@@ -236,7 +240,7 @@ class Vector:
         if isinstance(other, Vector):
             a=self._ar-other._ar
         else:
-            a=self._ar-array(other)
+            a=self._ar-numpy.array(other)
         return Vector(a)
 
     def __mul__(self, other):
@@ -245,20 +249,20 @@ class Vector:
 
     def __div__(self, x):
         "Return Vector(coords/a)"
-        a=self._ar/array(x)
+        a=self._ar/numpy.array(x)
         return Vector(a)
 
     def __pow__(self, other):
         "Return VectorxVector (cross product) or Vectorxscalar"
         if isinstance(other, Vector):
-            a,b,c=self._ar
-            d,e,f=other._ar
-            c1=determinant(array(((b,c), (e,f))))
-            c2=-determinant(array(((a,c), (d,f))))
-            c3=determinant(array(((a,b), (d,e))))
-            return Vector(c1,c2,c3)
+            a, b, c=self._ar
+            d, e, f=other._ar
+            c1=numpy.linalg.det(numpy.array(((b, c), (e, f))))
+            c2=-numpy.linalg.det(numpy.array(((a, c), (d, f))))
+            c3=numpy.linalg.det(numpy.array(((a, b), (d, e))))
+            return Vector(c1, c2, c3)
         else:
-            a=self._ar*array(other)
+            a=self._ar*numpy.array(other)
             return Vector(a)
 
     def __getitem__(self, i):
@@ -267,9 +271,12 @@ class Vector:
     def __setitem__(self, i, value):
         self._ar[i]=value
 
+    def __contains__(self, i):
+        return (i in self._ar)
+
     def norm(self):
         "Return vector norm"
-        return sqrt(sum(self._ar*self._ar))
+        return numpy.sqrt(sum(self._ar*self._ar))
 
     def normsq(self):
         "Return square of vector norm"
@@ -291,22 +298,22 @@ class Vector:
         n2=other.norm()
         c=(self*other)/(n1*n2)
         # Take care of roundoff errors
-        c=min(c,1)
-        c=max(-1,c)
-        return arccos(c)
+        c=min(c, 1)
+        c=max(-1, c)
+        return numpy.arccos(c)
 
     def get_array(self):
         "Return (a copy of) the array of coordinates"
-        return array(self._ar)
+        return numpy.array(self._ar)
 
     def left_multiply(self, matrix):
         "Return Vector=Matrix x Vector"
-        a=matrixmultiply(matrix, self._ar)
+        a=numpy.dot(matrix, self._ar)
         return Vector(a)
 
     def right_multiply(self, matrix):
         "Return Vector=Vector x Matrix"
-        a=matrixmultiply(self._ar, matrix)
+        a=numpy.dot(self._ar, matrix)
         return Vector(a)
 
     def copy(self):
@@ -315,72 +322,70 @@ class Vector:
 
 if __name__=="__main__":
 
-        from math import pi
-        from RandomArray import *
-        from Numeric import *
+    from numpy.random import random
 
-        v1=Vector(0,0,1)
-        v2=Vector(0,0,0)
-        v3=Vector(0,1,0)
-        v4=Vector(1,1,0)
+    v1=Vector(0, 0, 1)
+    v2=Vector(0, 0, 0)
+    v3=Vector(0, 1, 0)
+    v4=Vector(1, 1, 0)
 
-        v4.normalize()
+    v4.normalize()
 
-        print v4
+    print(v4)
 
-        print calc_angle(v1, v2, v3)
-        dih=calc_dihedral(v1, v2, v3, v4)
-        # Test dihedral sign
-        assert(dih>0)
-        print "DIHEDRAL ", dih
+    print(calc_angle(v1, v2, v3))
+    dih=calc_dihedral(v1, v2, v3, v4)
+    # Test dihedral sign
+    assert(dih>0)
+    print("DIHEDRAL %f" % dih)
 
-        ref=refmat(v1, v3)
-        rot=rotmat(v1, v3)
+    ref=refmat(v1, v3)
+    rot=rotmat(v1, v3)
 
-        print v3
-        print v1.left_multiply(ref)
-        print v1.left_multiply(rot)
-        print v1.right_multiply(transpose(rot))
+    print(v3)
+    print(v1.left_multiply(ref))
+    print(v1.left_multiply(rot))
+    print(v1.right_multiply(numpy.transpose(rot)))
 
-        # -
-        print v1-v2
-        print v1-1
-        print v1+(1,2,3)
-        # +
-        print v1+v2
-        print v1+3
-        print v1-(1,2,3)
-        # *
-        print v1*v2
-        # /
-        print v1/2
-        print v1/(1,2,3)
-        # **
-        print v1**v2
-        print v1**2
-        print v1**(1,2,3)
-        # norm
-        print v1.norm()
-        # norm squared
-        print v1.normsq()
-        # setitem
-        v1[2]=10
-        print v1
-        # getitem
-        print v1[2]
+    # -
+    print(v1-v2)
+    print(v1-1)
+    print(v1+(1, 2, 3))
+    # +
+    print(v1+v2)
+    print(v1+3)
+    print(v1-(1, 2, 3))
+    # *
+    print(v1*v2)
+    # /
+    print(v1/2)
+    print(v1/(1, 2, 3))
+    # **
+    print(v1**v2)
+    print(v1**2)
+    print(v1**(1, 2, 3))
+    # norm
+    print(v1.norm())
+    # norm squared
+    print(v1.normsq())
+    # setitem
+    v1[2]=10
+    print(v1)
+    # getitem
+    print(v1[2])
 
-        print array(v1)
+    print(numpy.array(v1))
 
-        print "ROT"
+    print("ROT")
 
-        angle=random()*pi
-        axis=Vector(random(3)-random(3))
-        axis.normalize()
+    angle=random()*numpy.pi
+    axis=Vector(random(3)-random(3))
+    axis.normalize()
 
-        m=rotaxis(angle, axis)
+    m=rotaxis(angle, axis)
 
-        cangle, caxis=m2rotaxis(m)
+    cangle, caxis=m2rotaxis(m)
 
-        print angle-cangle
-        print axis-caxis
-        print
+    print(angle-cangle)
+    print(axis-caxis)
+    print("")
