@@ -48,46 +48,49 @@ _open         -- Internally used function.
 
 """
 
+from __future__ import print_function
+
 import os
 import re
 
-import Des
-import Cla
-import Hie
-import Residues
+from . import Des
+from . import Cla
+from . import Hie
+from . import Residues
 from Bio import SeqIO
 from Bio.Seq import Seq
 
+__docformat__ = "restructuredtext en"
 
-nodeCodeDict = { 'cl':'class', 'cf':'fold', 'sf':'superfamily',
-                 'fa':'family', 'dm':'protein', 'sp':'species', 'px':'domain'}
+nodeCodeDict = {'cl': 'class', 'cf': 'fold', 'sf': 'superfamily',
+                'fa': 'family', 'dm': 'protein', 'sp': 'species', 'px': 'domain'}
 
 
-_nodetype_to_code= { 'class': 'cl', 'fold': 'cf', 'superfamily': 'sf',
-                     'family': 'fa', 'protein': 'dm', 'species': 'sp', 'domain': 'px'}
+_nodetype_to_code= {'class': 'cl', 'fold': 'cf', 'superfamily': 'sf',
+                    'family': 'fa', 'protein': 'dm', 'species': 'sp', 'domain': 'px'}
 
-nodeCodeOrder = [ 'ro', 'cl', 'cf', 'sf', 'fa', 'dm', 'sp', 'px' ]
+nodeCodeOrder = ['ro', 'cl', 'cf', 'sf', 'fa', 'dm', 'sp', 'px']
 
-astralBibIds = [10,20,25,30,35,40,50,70,90,95,100]
+astralBibIds = [10, 20, 25, 30, 35, 40, 50, 70, 90, 95, 100]
 
 astralEvs = [10, 5, 1, 0.5, 0.1, 0.05, 0.01, 0.005, 0.001, 1e-4, 1e-5, 1e-10, 1e-15,
              1e-20, 1e-25, 1e-50]
 
-astralEv_to_file = { 10: 'e+1', 5: 'e+0,7', 1: 'e+0', 0.5: 'e-0,3', 0.1: 'e-1',
-                     0.05: 'e-1,3', 0.01: 'e-2', 0.005: 'e-2,3', 0.001: 'e-3',
-                     1e-4: 'e-4',  1e-5: 'e-5', 1e-10: 'e-10', 1e-15: 'e-15',
-                     1e-20: 'e-20', 1e-25: 'e-25', 1e-50: 'e-50' }
+astralEv_to_file = {10: 'e+1', 5: 'e+0,7', 1: 'e+0', 0.5: 'e-0,3', 0.1: 'e-1',
+                    0.05: 'e-1,3', 0.01: 'e-2', 0.005: 'e-2,3', 0.001: 'e-3',
+                    1e-4: 'e-4', 1e-5: 'e-5', 1e-10: 'e-10', 1e-15: 'e-15',
+                    1e-20: 'e-20', 1e-25: 'e-25', 1e-50: 'e-50'}
 
-astralEv_to_sql = { 10: 'e1', 5: 'e0_7', 1: 'e0', 0.5: 'e_0_3', 0.1: 'e_1',
-                     0.05: 'e_1_3', 0.01: 'e_2', 0.005: 'e_2_3', 0.001: 'e_3',
-                     1e-4: 'e_4',  1e-5: 'e_5', 1e-10: 'e_10', 1e-15: 'e_15',
-                     1e-20: 'e_20', 1e-25: 'e_25', 1e-50: 'e_50' }
+astralEv_to_sql = {10: 'e1', 5: 'e0_7', 1: 'e0', 0.5: 'e_0_3', 0.1: 'e_1',
+                   0.05: 'e_1_3', 0.01: 'e_2', 0.005: 'e_2_3', 0.001: 'e_3',
+                   1e-4: 'e_4', 1e-5: 'e_5', 1e-10: 'e_10', 1e-15: 'e_15',
+                   1e-20: 'e_20', 1e-25: 'e_25', 1e-50: 'e_50'}
 
 try:
-    #See if the cmp function exists (will on Python 2)
+    # See if the cmp function exists (will on Python 2)
     _cmp = cmp
 except NameError:
-    def _cmp(a,b):
+    def _cmp(a, b):
         """Implementation of cmp(x,y) for Python 3 (PRIVATE).
 
         Based on Python 3 docs which say if you really need the cmp()
@@ -114,10 +117,10 @@ def cmp_sccs(sccs1, sccs2):
     if s1[0] != s2[0]:
         return _cmp(s1[0], s2[0])
 
-    s1 = list(map(int, s1[1:]))
-    s2 = list(map(int, s2[1:]))
+    s1 = [int(x) for x in s1[1:]]
+    s2 = [int(x) for x in s2[1:]]
 
-    return _cmp(s1,s2)
+    return _cmp(s1, s2)
 
 
 _domain_re = re.compile(r">?([\w_\.]*)\s+([\w\.]*)\s+\(([^)]*)\) (.*)")
@@ -153,8 +156,8 @@ def parse_domain(str):
 
 
 def _open_scop_file(scop_dir_path, version, filetype):
-    filename = "dir.%s.scop.txt_%s" % (filetype,version)
-    handle = open(os.path.join( scop_dir_path, filename))
+    filename = "dir.%s.scop.txt_%s" % (filetype, version)
+    handle = open(os.path.join(scop_dir_path, filename))
     return handle
 
 
@@ -203,14 +206,14 @@ class Scop(object):
                     if cla_handle or des_handle or hie_handle:
                         raise RuntimeError("Cannot specify SCOP directory and specific files")
 
-                    cla_handle = _open_scop_file( dir_path, version, 'cla')
-                    des_handle = _open_scop_file( dir_path, version, 'des')
-                    hie_handle = _open_scop_file( dir_path, version, 'hie')
+                    cla_handle = _open_scop_file(dir_path, version, 'cla')
+                    des_handle = _open_scop_file(dir_path, version, 'des')
+                    hie_handle = _open_scop_file(dir_path, version, 'hie')
 
                 root = Node()
                 domains = []
-                root.sunid=0
-                root.type='ro'
+                root.sunid = 0
+                root.type = 'ro'
                 sunidDict[root.sunid] = root
                 self.root = root
                 root.description = 'SCOP Root'
@@ -218,11 +221,11 @@ class Scop(object):
                 # Build the rest of the nodes using the DES file
                 records = Des.parse(des_handle)
                 for record in records:
-                    if record.nodetype =='px':
+                    if record.nodetype == 'px':
                         n = Domain()
                         n.sid = record.name
                         domains.append(n)
-                    else :
+                    else:
                         n = Node()
                     n.sunid = record.sunid
                     n.type = record.nodetype
@@ -235,11 +238,11 @@ class Scop(object):
                 records = Hie.parse(hie_handle)
                 for record in records:
                     if record.sunid not in sunidDict:
-                        print record.sunid
+                        print(record.sunid)
 
                     n = sunidDict[record.sunid]
 
-                    if record.parent != '' : # Not root node
+                    if record.parent != '': # Not root node
 
                         if record.parent not in sunidDict:
                             raise ValueError("Incomplete data?")
@@ -310,27 +313,21 @@ class Scop(object):
 
     def write_hie(self, handle):
         """Build an HIE SCOP parsable file from this object"""
-        nodes = self._sunidDict.values()
         # We order nodes to ease comparison with original file
-        nodes.sort(key = lambda n: n.sunid)
-        for n in nodes:
+        for n in sorted(self._sunidDict.values(), key=lambda n: n.sunid):
             handle.write(str(n.toHieRecord()))
 
     def write_des(self, handle):
         """Build a DES SCOP parsable file from this object"""
-        nodes = self._sunidDict.values()
         # Origional SCOP file is not ordered?
-        nodes.sort(key = lambda n: n.sunid)
-        for n in nodes:
+        for n in sorted(self._sunidDict.values(), key=lambda n: n.sunid):
             if n != self.root:
                 handle.write(str(n.toDesRecord()))
 
     def write_cla(self, handle):
         """Build a CLA SCOP parsable file from this object"""
-        nodes = self._sidDict.values()
         # We order nodes to ease comparison with original file
-        nodes.sort(key = lambda n: n.sunid)
-        for n in nodes:
+        for n in sorted(self._sidDict.values(), key=lambda n: n.sunid):
             handle.write(str(n.toClaRecord()))
 
     def getDomainFromSQL(self, sunid=None, sid=None):
@@ -353,7 +350,7 @@ class Scop(object):
         if data is not None:
             n = None
 
-            #determine if Node or Domain
+            # determine if Node or Domain
             if data[1] != "px":
                 n = Node(scop=self)
 
@@ -367,12 +364,12 @@ class Scop(object):
                 cur.execute("select sid, residues, pdbid from cla where sunid=%s",
                                sunid)
 
-                [n.sid,n.residues,pdbid] = cur.fetchone()
+                [n.sid, n.residues, pdbid] = cur.fetchone()
                 n.residues = Residues.Residues(n.residues)
                 n.residues.pdbid=pdbid
                 self._sidDict[n.sid] = n
 
-            [n.sunid,n.type,n.sccs,n.description] = data
+            [n.sunid, n.type, n.sccs, n.description] = data
 
             if data[1] != 'ro':
                 cur.execute("SELECT parent FROM hie WHERE child=%s", sunid)
@@ -408,7 +405,7 @@ class Scop(object):
         # SQL cla table knows nothing about 'ro'
         if node.type == 'ro':
             for c in node.getChildren():
-                for d in self.getDescendentsFromSQL(c,type):
+                for d in self.getDescendentsFromSQL(c, type):
                     des_list.append(d)
             return des_list
 
@@ -421,7 +418,7 @@ class Scop(object):
             for d in data:
                 if int(d[0]) not in self._sunidDict:
                     n = Node(scop=self)
-                    [n.sunid,n.type,n.sccs,n.description] = d
+                    [n.sunid, n.type, n.sccs, n.description] = d
                     n.sunid=int(n.sunid)
                     self._sunidDict[n.sunid] = n
 
@@ -434,7 +431,7 @@ class Scop(object):
                         children.append(c[0])
                     n.children = children
 
-                des_list.append( self._sunidDict[int(d[0])] )
+                des_list.append(self._sunidDict[int(d[0])])
 
         else:
             cur.execute("SELECT cla.sunid,sid,pdbid,residues,cla.sccs,type,description,sp\
@@ -445,9 +442,9 @@ class Scop(object):
             for d in data:
                 if int(d[0]) not in self._sunidDict:
                     n = Domain(scop=self)
-                    #[n.sunid, n.sid, n.pdbid, n.residues, n.sccs, n.type,
-                    #n.description,n.parent] = data
-                    [n.sunid,n.sid, pdbid,n.residues,n.sccs,n.type,n.description,
+                    # [n.sunid, n.sid, n.pdbid, n.residues, n.sccs, n.type,
+                    # n.description,n.parent] = data
+                    [n.sunid, n.sid, pdbid, n.residues, n.sccs, n.type, n.description,
                      n.parent] = d[0:8]
                     n.residues = Residues.Residues(n.residues)
                     n.residues.pdbid = pdbid
@@ -455,7 +452,7 @@ class Scop(object):
                     self._sunidDict[n.sunid] = n
                     self._sidDict[n.sid] = n
 
-                des_list.append( self._sunidDict[int(d[0])] )
+                des_list.append(self._sunidDict[int(d[0])])
 
         return des_list
 
@@ -467,7 +464,7 @@ class Scop(object):
         cur.execute("CREATE TABLE hie (parent INT, child INT, PRIMARY KEY (child),\
         INDEX (parent) )")
 
-        for p in self._sunidDict.itervalues():
+        for p in self._sunidDict.values():
             for c in p.children:
                 cur.execute("INSERT INTO hie VALUES (%s,%s)" % (p.sunid, c.sunid))
 
@@ -480,14 +477,14 @@ class Scop(object):
         residues VARCHAR(50), sccs CHAR(10), cl INT, cf INT, sf INT, fa INT,\
         dm INT, sp INT, px INT, PRIMARY KEY (sunid), INDEX (SID) )")
 
-        for n in self._sidDict.itervalues():
+        for n in self._sidDict.values():
             c = n.toClaRecord()
-            cur.execute( "INSERT INTO cla VALUES (%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)",
+            cur.execute("INSERT INTO cla VALUES (%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)",
                          (n.sunid, n.sid, c.residues.pdbid, c.residues, n.sccs,
                          n.getAscendent('cl').sunid, n.getAscendent('cf').sunid,
                          n.getAscendent('sf').sunid, n.getAscendent('fa').sunid,
                          n.getAscendent('dm').sunid, n.getAscendent('sp').sunid,
-                         n.sunid ))
+                         n.sunid))
 
     def write_des_sql(self, handle):
         """Write DES data to SQL database"""
@@ -498,9 +495,9 @@ class Scop(object):
         description VARCHAR(255),\
         PRIMARY KEY (sunid) )")
 
-        for n in self._sunidDict.itervalues():
-            cur.execute( "INSERT INTO des VALUES (%s,%s,%s,%s)",
-                         ( n.sunid, n.type, n.sccs, n.description ) )
+        for n in self._sunidDict.values():
+            cur.execute("INSERT INTO des VALUES (%s,%s,%s,%s)",
+                         (n.sunid, n.type, n.sccs, n.description))
 
 
 class Node(object):
@@ -524,13 +521,13 @@ class Node(object):
         constructor, this will be used to lookup related references using the SQL
         methods.  If no instance is provided, it is assumed the whole tree exists
         and is connected."""
-        self.sunid=''
+        self.sunid = ''
         self.parent = None
-        self.children=[]
+        self.children = []
         self.sccs = ''
-        self.type =''
-        self.description =''
-        self.scop=scop
+        self.type = ''
+        self.description = ''
+        self.scop = scop
 
     def __str__(self):
         s = []
@@ -567,16 +564,16 @@ class Node(object):
         if self.scop is None:
             return self.children
         else:
-            return map( self.scop.getNodeBySunid, self.children )
+            return [self.scop.getNodeBySunid(x) for x in self.children]
 
     def getParent(self):
         """Return the parent of this Node"""
         if self.scop is None:
             return self.parent
         else:
-            return self.scop.getNodeBySunid( self.parent )
+            return self.scop.getNodeBySunid(self.parent)
 
-    def getDescendents( self, node_type):
+    def getDescendents(self, node_type):
         """ Return a list of all decendent nodes of the given type. Node type can a
         two letter code or longer description. e.g. 'fa' or 'family'
         """
@@ -585,26 +582,26 @@ class Node(object):
 
         nodes = [self]
         if self.scop:
-            return self.scop.getDescendentsFromSQL(self,node_type)
+            return self.scop.getDescendentsFromSQL(self, node_type)
         while nodes[0].type != node_type:
             if nodes[0].type == 'px':
                 return [] # Fell of the bottom of the hierarchy
             child_list = []
             for n in nodes:
                 for child in n.getChildren():
-                    child_list.append( child )
+                    child_list.append(child)
                 nodes = child_list
 
         return nodes
 
-    def getAscendent( self, node_type):
+    def getAscendent(self, node_type):
         """ Return the ancenstor node of the given type, or None.Node type can a
         two letter code or longer description. e.g. 'fa' or 'family'"""
         if node_type in _nodetype_to_code:
             node_type = _nodetype_to_code[node_type]
 
         if self.scop:
-            return self.scop.getAscendentFromSQL(self,node_type)
+            return self.scop.getAscendentFromSQL(self, node_type)
         else:
             n = self
             if n.type == node_type:
@@ -621,13 +618,13 @@ class Node(object):
 class Domain(Node):
     """ A SCOP domain. A leaf node in the Scop hierarchy.
 
-    sid      -- The SCOP domain identifier. e.g. 'd5hbib_'
+        - sid      -- The SCOP domain identifier. e.g. ``"d5hbib_"``
 
-    residues -- A Residue object. It defines the collection
-                  of PDB atoms that make up this domain.
+        - residues -- A Residue object. It defines the collection
+          of PDB atoms that make up this domain.
     """
-    def __init__(self,scop=None):
-        Node.__init__(self,scop=scop)
+    def __init__(self, scop=None):
+        Node.__init__(self, scop=scop)
         self.sid = ''
         self.residues = None
 
@@ -668,7 +665,7 @@ class Domain(Node):
 
         # Order does not matter in the hierarchy field. For more info, see
         # http://scop.mrc-lmb.cam.ac.uk/scop/release-notes.html
-        #rec.hierarchy.reverse()
+        # rec.hierarchy.reverse()
 
         return rec
 
@@ -678,26 +675,26 @@ class Astral(object):
     as well as clusterings by percent id or evalue.
     """
 
-    def __init__( self, dir_path=None, version=None, scop=None,
+    def __init__(self, dir_path=None, version=None, scop=None,
                   astral_file=None, db_handle=None):
         """
         Initialise the astral database.
 
         You must provide either a directory of SCOP files:
 
-        dir_path - string, the path to location of the scopseq-x.xx directory
-                   (not the directory itself), and
-        version   -a version number.
+            - dir_path - string, the path to location of the scopseq-x.xx directory
+                       (not the directory itself), and
+            - version   -a version number.
 
         or, a FASTA file:
 
-        astral_file - string, a path to a fasta file (which will be loaded in memory)
+            - astral_file - string, a path to a fasta file (which will be loaded in memory)
 
         or, a MYSQL database:
 
-        db_handle - a database handle for a MYSQL database containing a table
-                    'astral' with the astral data in it.  This can be created
-                    using writeToSQL.
+            - db_handle - a database handle for a MYSQL database containing a table
+              'astral' with the astral data in it.  This can be created
+              using writeToSQL.
         """
 
         if astral_file is None and dir_path is None and db_handle is None:
@@ -714,12 +711,12 @@ class Astral(object):
                 raise RuntimeError("must provide dir_path and version")
 
             self.version = version
-            self.path = os.path.join( dir_path, "scopseq-%s" % version)
+            self.path = os.path.join(dir_path, "scopseq-%s" % version)
             astral_file = "astral-scopdom-seqres-all-%s.fa" % self.version
             astral_file = os.path.join(self.path, astral_file)
 
         if astral_file:
-            #Build a dictionary of SeqRecord objects in the FASTA file, IN MEMORY
+            # Build a dictionary of SeqRecord objects in the FASTA file, IN MEMORY
             self.fasta_dict = SeqIO.to_dict(SeqIO.parse(astral_file, "fasta"))
 
         self.astral_file = astral_file
@@ -728,7 +725,7 @@ class Astral(object):
         self.IdDatasets = {}
         self.IdDatahash = {}
 
-    def domainsClusteredByEv(self,id):
+    def domainsClusteredByEv(self, id):
         """get domains clustered by evalue"""
         if id not in self.EvDatasets:
             if self.db_handle:
@@ -739,13 +736,13 @@ class Astral(object):
                     raise RuntimeError("No scopseq directory specified")
 
                 file_prefix = "astral-scopdom-seqres-sel-gs"
-                filename = "%s-e100m-%s-%s.id" % (file_prefix, astralEv_to_file[id] ,
+                filename = "%s-e100m-%s-%s.id" % (file_prefix, astralEv_to_file[id],
                                                   self.version)
-                filename = os.path.join(self.path,filename)
+                filename = os.path.join(self.path, filename)
                 self.EvDatasets[id] = self.getAstralDomainsFromFile(filename)
         return self.EvDatasets[id]
 
-    def domainsClusteredById(self,id):
+    def domainsClusteredById(self, id):
         """get domains clustered by percent id"""
         if id not in self.IdDatasets:
             if self.db_handle:
@@ -756,18 +753,18 @@ class Astral(object):
 
                 file_prefix = "astral-scopdom-seqres-sel-gs"
                 filename = "%s-bib-%s-%s.id" % (file_prefix, id, self.version)
-                filename = os.path.join(self.path,filename)
+                filename = os.path.join(self.path, filename)
                 self.IdDatasets[id] = self.getAstralDomainsFromFile(filename)
         return self.IdDatasets[id]
 
-    def getAstralDomainsFromFile(self,filename=None,file_handle=None):
+    def getAstralDomainsFromFile(self, filename=None, file_handle=None):
         """Get the scop domains from a file containing a list of sids"""
         if file_handle is None and filename is None:
             raise RuntimeError("You must provide a filename or handle")
         if not file_handle:
             file_handle = open(filename)
         doms = []
-        while 1:
+        while True:
             line = file_handle.readline()
             if not line:
                 break
@@ -776,8 +773,8 @@ class Astral(object):
         if filename:
             file_handle.close()
 
-        doms = filter( lambda a: a[0]=='d', doms )
-        doms = map( self.scop.getDomainBySid, doms )
+        doms = [a for a in doms if a[0]=='d']
+        doms = [self.scop.getDomainBySid(x) for x in doms]
         return doms
 
     def getAstralDomainsFromSQL(self, column):
@@ -786,11 +783,11 @@ class Astral(object):
         cur = self.db_handle.cursor()
         cur.execute("SELECT sid FROM astral WHERE "+column+"=1")
         data = cur.fetchall()
-        data = map( lambda x: self.scop.getDomainBySid(x[0]), data)
+        data = [self.scop.getDomainBySid(x[0]) for x in data]
 
         return data
 
-    def getSeqBySid(self,domain):
+    def getSeqBySid(self, domain):
         """get the seq record of a given domain from its sid"""
         if self.db_handle is None:
             return self.fasta_dict[domain].seq
@@ -799,11 +796,11 @@ class Astral(object):
             cur.execute("SELECT seq FROM astral WHERE sid=%s", domain)
             return Seq(cur.fetchone()[0])
 
-    def getSeq(self,domain):
+    def getSeq(self, domain):
         """Return seq associated with domain"""
         return self.getSeqBySid(domain.sid)
 
-    def hashedDomainsById(self,id):
+    def hashedDomainsById(self, id):
         """Get domains clustered by sequence identity in a dict"""
         if id not in self.IdDatahash:
             self.IdDatahash[id] = {}
@@ -811,7 +808,7 @@ class Astral(object):
                 self.IdDatahash[id][d] = 1
         return self.IdDatahash[id]
 
-    def hashedDomainsByEv(self,id):
+    def hashedDomainsByEv(self, id):
         """Get domains clustered by evalue in a dict"""
         if id not in self.EvDatahash:
             self.EvDatahash[id] = {}
@@ -819,11 +816,11 @@ class Astral(object):
                 self.EvDatahash[id][d] = 1
         return self.EvDatahash[id]
 
-    def isDomainInId(self,dom,id):
+    def isDomainInId(self, dom, id):
         """Returns true if the domain is in the astral clusters for percent ID"""
         return dom in self.hashedDomainsById(id)
 
-    def isDomainInEv(self,dom,id):
+    def isDomainInEv(self, dom, id):
         """Returns true if the domain is in the ASTRAL clusters for evalues"""
         return dom in self.hashedDomainsByEv(id)
 
@@ -865,10 +862,10 @@ def search(pdb=None, key=None, sid=None, disp=None, dir=None, loc=None,
     Raises an IOError if there's a network error.
 
     """
-    params = {'pdb' : pdb, 'key' : key, 'sid' : sid, 'disp' : disp,
-              'dir' : dir, 'loc' : loc}
+    params = {'pdb': pdb, 'key': key, 'sid': sid, 'disp': disp,
+              'dir': dir, 'loc': loc}
     variables = {}
-    for k, v in params.iteritems():
+    for k, v in params.items():
         if v is not None:
             variables[k] = v
     variables.update(keywds)
@@ -884,17 +881,14 @@ def _open(cgi, params={}, get=1):
     simple error checking, and will raise an IOError if it encounters one.
 
     """
-    import urllib
-    import urllib2
+    from Bio._py3k import urlopen, urlencode
+
     # Open a handle to SCOP.
-    options = urllib.urlencode(params)
-    try:
-        if get:  # do a GET
-            if options:
-                cgi += "?" + options
-            handle = urllib2.urlopen(cgi)
-        else:    # do a POST
-            handle = urllib2.urlopen(cgi, data=options)
-    except urllib2.HTTPError, exception:
-        raise exception
+    options = urlencode(params)
+    if get:  # do a GET
+        if options:
+            cgi += "?" + options
+        handle = urlopen(cgi)
+    else:    # do a POST
+        handle = urlopen(cgi, data=options)
     return handle
