@@ -2,6 +2,8 @@
 # Created: Wed May 29 08:07:18 2002
 # thomas@cbs.dtu.dk, Cecilia.Alsmark@ebc.uu.se
 # Copyright 2001 by Thomas Sicheritz-Ponten and Cecilia Alsmark.
+# Revisions copyright 2014 by Markus Piotrowski.
+# Revisions copyright 2014 by Peter Cock.
 # All rights reserved.
 # This code is part of the Biopython distribution and governed by its
 # license.  Please see the LICENSE file that should have been included
@@ -9,13 +11,17 @@
 
 """Miscellaneous functions for dealing with sequences."""
 
+from __future__ import print_function
+
 import re
 from math import pi, sin, cos
 
-from Bio.Seq import Seq
+from Bio.Seq import Seq, MutableSeq
+from Bio import Alphabet
 from Bio.Alphabet import IUPAC
 from Bio.Data import IUPACData
 
+__docformat__ = "restructuredtext en"
 
 ######################################
 # DNA
@@ -37,7 +43,7 @@ def GC(seq):
     Note that this will return zero for an empty sequence.
     """
     try:
-        gc = sum(map(seq.count, ['G', 'C', 'g', 'c', 'S', 's']))
+        gc = sum(seq.count(x) for x in ['G', 'C', 'g', 'c', 'S', 's'])
         return gc*100.0/len(seq)
     except ZeroDivisionError:
         return 0.0
@@ -107,20 +113,23 @@ def GC_skew(seq, window=100):
 def xGC_skew(seq, window=1000, zoom=100,
                          r=300, px=100, py=100):
     """Calculates and plots normal and accumulated GC skew (GRAPHICS !!!)."""
-    from Tkinter import Scrollbar, Canvas, BOTTOM, BOTH, ALL, \
-                        VERTICAL, HORIZONTAL, RIGHT, LEFT, X, Y
-    yscroll = Scrollbar(orient=VERTICAL)
-    xscroll = Scrollbar(orient=HORIZONTAL)
-    canvas = Canvas(yscrollcommand=yscroll.set,
-                    xscrollcommand=xscroll.set, background='white')
+    try:
+        import Tkinter as tkinter # Python 2
+    except ImportError:
+        import tkinter # Python 3
+
+    yscroll = tkinter.Scrollbar(orient=tkinter.VERTICAL)
+    xscroll = tkinter.Scrollbar(orient=tkinter.HORIZONTAL)
+    canvas = tkinter.Canvas(yscrollcommand=yscroll.set,
+                            xscrollcommand=xscroll.set, background='white')
     win = canvas.winfo_toplevel()
     win.geometry('700x700')
 
     yscroll.config(command=canvas.yview)
     xscroll.config(command=canvas.xview)
-    yscroll.pack(side=RIGHT, fill=Y)
-    xscroll.pack(side=BOTTOM, fill=X)
-    canvas.pack(fill=BOTH, side=LEFT, expand=1)
+    yscroll.pack(side=tkinter.RIGHT, fill=tkinter.Y)
+    xscroll.pack(side=tkinter.BOTTOM, fill=tkinter.X)
+    canvas.pack(fill=tkinter.BOTH, side=tkinter.LEFT, expand=1)
     canvas.update()
 
     X0, Y0 = r + px, r + py
@@ -162,15 +171,7 @@ def xGC_skew(seq, window=1000, zoom=100,
         canvas.update()
         start += window
 
-    canvas.configure(scrollregion=canvas.bbox(ALL))
-
-
-def molecular_weight(seq):
-    """Calculate the molecular weight of a DNA sequence."""
-    if isinstance(seq, str):
-        seq = Seq(seq, IUPAC.unambiguous_dna)
-    weight_table = IUPACData.unambiguous_dna_weights
-    return sum(weight_table[x] for x in seq)
+    canvas.configure(scrollregion=canvas.bbox(tkinter.ALL))
 
 
 def nt_search(seq, subseq):
@@ -207,6 +208,7 @@ def nt_search(seq, subseq):
 ######################
 # {{{
 
+
 def seq3(seq, custom_map={'*': 'Ter'}, undef_code='Xaa'):
     """Turn a one letter code protein sequence into one with three letter codes.
 
@@ -247,11 +249,11 @@ def seq3(seq, custom_map={'*': 'Ter'}, undef_code='Xaa'):
     """
     # not doing .update() on IUPACData dict with custom_map dict
     # to preserve its initial state (may be imported in other modules)
-    threecode = dict(IUPACData.protein_letters_1to3_extended.items() +
-            custom_map.items())
-    #We use a default of 'Xaa' for undefined letters
-    #Note this will map '-' to 'Xaa' which may be undesirable!
-    return ''.join([threecode.get(aa, undef_code) for aa in seq])
+    threecode = dict(list(IUPACData.protein_letters_1to3_extended.items()) +
+                     list(custom_map.items()))
+    # We use a default of 'Xaa' for undefined letters
+    # Note this will map '-' to 'Xaa' which may be undesirable!
+    return ''.join(threecode.get(aa, undef_code) for aa in seq)
 
 
 def seq1(seq, custom_map={'Ter': '*'}, undef_code='X'):
@@ -300,11 +302,11 @@ def seq1(seq, custom_map={'Ter': '*'}, undef_code='X'):
     # reverse map of threecode
     # upper() on all keys to enable caps-insensitive input seq handling
     onecode = dict((k.upper(), v) for k, v in
-            IUPACData.protein_letters_3to1_extended.items())
+                   IUPACData.protein_letters_3to1_extended.items())
     # add the given termination codon code and custom maps
-    onecode.update((k.upper(), v) for (k, v) in custom_map.iteritems())
+    onecode.update((k.upper(), v) for (k, v) in custom_map.items())
     seqlist = [seq[3*i:3*(i+1)] for i in range(len(seq) // 3)]
-    return ''.join([onecode.get(aa.upper(), undef_code) for aa in seqlist])
+    return ''.join(onecode.get(aa.upper(), undef_code) for aa in seqlist)
 
 
 # }}}
@@ -315,6 +317,139 @@ def seq1(seq, custom_map={'Ter': '*'}, undef_code='X'):
 # {{{
 
 
+def molecular_weight(seq, seq_type=None, double_stranded=False, circular=False,
+                     monoisotopic=False):
+    """Calculates the molecular weight of a DNA, RNA or protein sequence.
+
+    Only unambiguous letters are allowed. Nucleotide sequences are assumed to
+    have a 5' phosphate.
+
+        - seq: String or Biopython sequence object.
+        - seq_type: The default (None) is to take the alphabet from the seq argument,
+          or assume DNA if the seq argument is a string. Override this with
+          a string 'DNA', 'RNA', or 'protein'.
+        - double_stranded: Calculate the mass for the double stranded molecule?
+        - circular: Is the molecule circular (has no ends)?
+        - monoisotopic: Use the monoisotopic mass tables?
+
+    Note that for backwards compatibility, if the seq argument is a string,
+    or Seq object with a generic alphabet, and no seq_type is specified
+    (i.e. left as None), then DNA is assumed.
+
+    >>> print("%0.2f" % molecular_weight("AGC"))
+    949.61
+    >>> print("%0.2f" % molecular_weight(Seq("AGC")))
+    949.61
+
+    However, it is better to be explicit - for example with strings:
+
+    >>> print("%0.2f" % molecular_weight("AGC", "DNA"))
+    949.61
+    >>> print("%0.2f" % molecular_weight("AGC", "RNA"))
+    997.61
+    >>> print("%0.2f" % molecular_weight("AGC", "protein"))
+    249.29
+
+    Or, with the sequence alphabet:
+
+    >>> from Bio.Seq import Seq
+    >>> from Bio.Alphabet import generic_dna, generic_rna, generic_protein
+    >>> print("%0.2f" % molecular_weight(Seq("AGC", generic_dna)))
+    949.61
+    >>> print("%0.2f" % molecular_weight(Seq("AGC", generic_rna)))
+    997.61
+    >>> print("%0.2f" % molecular_weight(Seq("AGC", generic_protein)))
+    249.29
+
+    Also note that contradictory sequence alphabets and seq_type will also
+    give an exception:
+
+    >>> from Bio.Seq import Seq
+    >>> from Bio.Alphabet import generic_dna
+    >>> print("%0.2f" % molecular_weight(Seq("AGC", generic_dna), "RNA"))
+    Traceback (most recent call last):
+      ...
+    ValueError: seq_type='RNA' contradicts DNA from seq alphabet
+
+    """
+    # Rewritten by Markus Piotrowski, 2014
+
+    # Find the alphabet type
+    tmp_type = ''
+    if isinstance(seq, Seq) or isinstance(seq, MutableSeq):
+        base_alphabet = Alphabet._get_base_alphabet(seq.alphabet)
+        if isinstance(base_alphabet, Alphabet.DNAAlphabet):
+            tmp_type = 'DNA'
+        elif isinstance(base_alphabet, Alphabet.RNAAlphabet):
+            tmp_type = 'RNA'
+        elif isinstance(base_alphabet, Alphabet.ProteinAlphabet):
+            tmp_type = 'protein'
+        elif isinstance(base_alphabet, Alphabet.ThreeLetterProtein):
+            tmp_type = 'protein'
+            # Convert to one-letter sequence. Have to use a string for seq1
+            seq = Seq(seq1(str(seq)), alphabet=Alphabet.ProteinAlphabet())
+        elif not isinstance(base_alphabet, Alphabet.Alphabet):
+            raise TypeError("%s is not a valid alphabet for mass calculations"
+                             % base_alphabet)
+        else:
+            tmp_type = "DNA" # backward compatibity
+        if seq_type and tmp_type and tmp_type != seq_type:
+            raise ValueError("seq_type=%r contradicts %s from seq alphabet"
+                             % (seq_type, tmp_type))
+        seq_type = tmp_type
+    elif isinstance(seq, str):
+        if seq_type is None:
+            seq_type = "DNA" # backward compatibity
+    else:
+        raise TypeError("Expected a string or Seq object, not seq=%r" % seq)
+
+    seq = ''.join(str(seq).split()).upper() # Do the minimum formatting
+
+    if seq_type == 'DNA':
+        if monoisotopic:
+            weight_table = IUPACData.monoisotopic_unambiguous_dna_weights
+        else:
+            weight_table = IUPACData.unambiguous_dna_weights
+    elif seq_type == 'RNA':
+        if monoisotopic:
+            weight_table = IUPACData.monoisotopic_unambiguous_rna_weights
+        else:
+            weight_table = IUPACData.unambiguous_rna_weights
+    elif seq_type == 'protein':
+        if monoisotopic:
+            weight_table = IUPACData.monoisotopic_protein_weights
+        else:
+            weight_table = IUPACData.protein_weights
+    else:
+        raise ValueError("Allowed seq_types are DNA, RNA or protein, not %r"
+                         % seq_type)
+
+    if monoisotopic:
+        water = 18.010565
+    else:
+        water = 18.0153
+
+    try:
+        weight = sum(weight_table[x] for x in seq) - (len(seq)-1) * water
+        if circular:
+            weight -= water
+    except KeyError as e:
+        raise ValueError('%s is not a valid unambiguous letter for %s'
+                         %(e, seq_type))
+    except:
+        raise
+
+    if seq_type in ('DNA', 'RNA') and double_stranded:
+        seq = str(Seq(seq).complement())
+        weight += sum(weight_table[x] for x in seq) - (len(seq)-1) * water
+        if circular:
+            weight -= water
+    elif seq_type == 'protein' and double_stranded:
+        raise ValueError('double-stranded proteins await their discovery')
+
+    return weight
+
+
 def six_frame_translations(seq, genetic_code=1):
     """Formatted string showing the 6 frame translations and GC content.
 
@@ -322,7 +457,7 @@ def six_frame_translations(seq, genetic_code=1):
     similar to DNA Striders six-frame translation
 
     >>> from Bio.SeqUtils import six_frame_translations
-    >>> print six_frame_translations("AUGGCCAUUGUAAUGGGCCGCUGA")
+    >>> print(six_frame_translations("AUGGCCAUUGUAAUGGGCCGCUGA"))
     GC_Frame: a:5 t:0 g:8 c:5 
     Sequence: auggccauug ... gggccgcuga, 24 nt, 54.17 %GC
     <BLANKLINE>
@@ -367,16 +502,16 @@ def six_frame_translations(seq, genetic_code=1):
         csubseq = comp[i:i+60]
         p = i//3
         res += '%d/%d\n' % (i+1, i/3+1)
-        res += '  ' + '  '.join(map(None, frames[3][p:p+20])) + '\n'
-        res += ' ' + '  '.join(map(None, frames[2][p:p+20])) + '\n'
-        res += '  '.join(map(None, frames[1][p:p+20])) + '\n'
+        res += '  ' + '  '.join(frames[3][p:p+20]) + '\n'
+        res += ' ' + '  '.join(frames[2][p:p+20]) + '\n'
+        res += '  '.join(frames[1][p:p+20]) + '\n'
         # seq
         res += subseq.lower() + '%5d %%\n' % int(GC(subseq))
         res += csubseq.lower() + '\n'
         # - frames
-        res += '  '.join(map(None, frames[-2][p:p+20])) +' \n'
-        res += ' ' + '  '.join(map(None, frames[-1][p:p+20])) + '\n'
-        res += '  ' + '  '.join(map(None, frames[-3][p:p+20])) + '\n\n'
+        res += '  '.join(frames[-2][p:p+20]) +' \n'
+        res += ' ' + '  '.join(frames[-1][p:p+20]) + '\n'
+        res += '  ' + '  '.join(frames[-3][p:p+20]) + '\n\n'
     return res
 
 # }}}
@@ -388,7 +523,7 @@ def six_frame_translations(seq, genetic_code=1):
 
 
 def quick_FASTA_reader(file):
-    """Simple FASTA reader, returning a list of string tuples (OBSOLETE).
+    """Simple FASTA reader, returning a list of string tuples (DEPRECATED).
 
     The single argument 'file' should be the filename of a FASTA format file.
     This function will open and read in the entire file, constructing a list
@@ -397,7 +532,7 @@ def quick_FASTA_reader(file):
 
     >>> seqs = quick_FASTA_reader("Fasta/dups.fasta")
     >>> for title, sequence in seqs:
-    ...     print title, sequence
+    ...     print("%s %s" % (title, sequence))
     alpha ACGTA
     beta CGTC
     gamma CCGCC
@@ -417,12 +552,15 @@ def quick_FASTA_reader(file):
     If you want to use simple strings, use the function SimpleFastaParser
     added to Bio.SeqIO.FastaIO in Biopython 1.61 instead.
     """
-    handle = open(file)
-    entries = []
+    import warnings
+    from Bio import BiopythonDeprecationWarning
+    warnings.warn("The quick_FASTA_reader has been deprecated and will be "
+                  "removed in a future release of Biopython. Please try "
+                  "function SimpleFastaParser from Bio.SeqIO.FastaIO "
+                  "instead.", BiopythonDeprecationWarning)
     from Bio.SeqIO.FastaIO import SimpleFastaParser
-    for title, sequence in SimpleFastaParser(handle):
-        entries.append((title, sequence))
-    handle.close()
+    with open(file) as handle:
+        entries = list(SimpleFastaParser(handle))
     return entries
 
 
@@ -434,21 +572,21 @@ def _test():
     import os
     import doctest
     if os.path.isdir(os.path.join("..", "Tests")):
-        print "Running doctests..."
+        print("Running doctests...")
         cur_dir = os.path.abspath(os.curdir)
         os.chdir(os.path.join("..", "Tests"))
         doctest.testmod()
         os.chdir(cur_dir)
         del cur_dir
-        print "Done"
+        print("Done")
     elif os.path.isdir(os.path.join("Tests")):
-        print "Running doctests..."
+        print("Running doctests...")
         cur_dir = os.path.abspath(os.curdir)
         os.chdir(os.path.join("Tests"))
         doctest.testmod()
         os.chdir(cur_dir)
         del cur_dir
-        print "Done"
+        print("Done")
 
 
 if __name__ == "__main__":
