@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-# Copyright 2010-2011 by Peter Cock.
+# Copyright 2010-2013 by Peter Cock.
 # All rights reserved.
 # This code is part of the Biopython distribution and governed by its
 # license.  Please see the LICENSE file that should have been included
@@ -70,8 +70,8 @@ how big it is from this 'BC' header, and thus seek immediately to
 the second block, and so on.
 
 The BAM indexing scheme records read positions using a 64 bit
-'virtual offset', comprising coffset<<16|uoffset, where coffset is
-the file offset of the BGZF block containing the start of the read
+'virtual offset', comprising coffset << 16 | uoffset, where coffset
+is the file offset of the BGZF block containing the start of the read
 (unsigned integer using up to 64-16 = 48 bits), and uoffset is the
 offset within the (decompressed) block (unsigned 16 bit integer).
 
@@ -87,30 +87,34 @@ CPU load).
 Warning about namespaces
 ------------------------
 
-It is considered a bad idea to use "from XXX import *" in Python, because
+It is considered a bad idea to use "from XXX import ``*``" in Python, because
 it pollutes the namespace. This is a real issue with Bio.bgzf (and the
 standard Python library gzip) because they contain a function called open
 i.e. Suppose you do this:
 
 >>> from Bio.bgzf import *
->>> print open.__module__
+>>> print(open.__module__)
 Bio.bgzf
 
 Or,
 
 >>> from gzip import *
->>> print open.__module__
+>>> print(open.__module__)
 gzip
 
 Notice that the open function has been replaced. You can "fix" this if you
 need to by importing the built-in open function:
 
->>> from __builtin__ import open
+>>> try:
+...     from __builtin__ import open # Python 2
+... except ImportError:
+...     from builtins import open # Python 3
+...
 
 However, what we recommend instead is to use the explicit namespace, e.g.
 
 >>> from Bio import bgzf
->>> print bgzf.open.__module__
+>>> print(bgzf.open.__module__)
 Bio.bgzf
 
 
@@ -136,20 +140,20 @@ attention to the file offsets which will be explained below:
 
 >>> handle = BgzfReader("GenBank/NC_000932.gb.bgz", "r")
 >>> assert 0 == handle.tell()
->>> print handle.readline().rstrip()
+>>> print(handle.readline().rstrip())
 LOCUS       NC_000932             154478 bp    DNA     circular PLN 15-APR-2009
 >>> assert 80 == handle.tell()
->>> print handle.readline().rstrip()
+>>> print(handle.readline().rstrip())
 DEFINITION  Arabidopsis thaliana chloroplast, complete genome.
 >>> assert 143 == handle.tell()
 >>> data = handle.read(70000)
 >>> assert 987828735 == handle.tell()
->>> print handle.readline().rstrip()
+>>> print(handle.readline().rstrip())
 f="GeneID:844718"
->>> print handle.readline().rstrip()
+>>> print(handle.readline().rstrip())
      CDS             complement(join(84337..84771,85454..85843))
 >>> offset = handle.seek(make_virtual_offset(55074, 126))
->>> print handle.readline().rstrip()
+>>> print(handle.readline().rstrip())
     68521 tatgtcattc gaaattgtat aaagacaact cctatttaat agagctattt gtgcaagtat
 >>> handle.close()
 
@@ -158,7 +162,7 @@ brings us to the key point about BGZF, which is the block structure:
 
 >>> handle = open("GenBank/NC_000932.gb.bgz", "rb")
 >>> for values in BgzfBlocks(handle):
-...     print "Raw start %i, raw length %i; data start %i, data length %i" % values
+...     print("Raw start %i, raw length %i; data start %i, data length %i" % values)
 Raw start 0, raw length 15073; data start 0, data length 65536
 Raw start 15073, raw length 17857; data start 65536, data length 65536
 Raw start 32930, raw length 22144; data start 131072, data length 65536
@@ -175,7 +179,7 @@ it is possible your BGZF file is incomplete.
 
 By reading ahead 70,000 bytes we moved into the second BGZF block,
 and at that point the BGZF virtual offsets start to look different
-a simple offset into the decompressed data as exposed by the gzip
+to a simple offset into the decompressed data as exposed by the gzip
 library.
 
 As an example, consider seeking to the decompressed position 196734.
@@ -187,9 +191,9 @@ does not always hold) and starting at byte 126 of the fourth block
 offset of 55074 and the offset within the block of 126 to get the
 BGZF virtual offset.
 
->>> print 55074 << 16 | 126
+>>> print(55074 << 16 | 126)
 3609329790
->>> print bgzf.make_virtual_offset(55074, 126)
+>>> print(bgzf.make_virtual_offset(55074, 126))
 3609329790
 
 Thus for this BGZF file, decompressed position 196734 corresponds
@@ -216,27 +220,28 @@ you want to index BGZF compressed sequence files:
 >>> handle = BgzfReader("GenBank/NC_000932.gb.bgz")
 >>> record = SeqIO.read(handle, "genbank")
 >>> handle.close()
->>> print record.id
+>>> print(record.id)
 NC_000932.1
 
 """
 
+from __future__ import print_function
+
+import sys  # to detect when under Python 2
 import zlib
 import struct
-import __builtin__  # to access the usual open function
 
 from Bio._py3k import _as_bytes, _as_string
+from Bio._py3k import open as _open
 
-#For Python 2 can just use: _bgzf_magic = '\x1f\x8b\x08\x04'
-#but need to use bytes on Python 3
-_bgzf_magic = _as_bytes("\x1f\x8b\x08\x04")
-_bgzf_header = _as_bytes("\x1f\x8b\x08\x04\x00\x00\x00\x00"
-                         "\x00\xff\x06\x00\x42\x43\x02\x00")
-_bgzf_eof = _as_bytes("\x1f\x8b\x08\x04\x00\x00\x00\x00\x00\xff\x06\x00BC" +
-                      "\x02\x00\x1b\x00\x03\x00\x00\x00\x00\x00\x00\x00\x00\x00")
-_bytes_BC = _as_bytes("BC")
-_empty_bytes_string = _as_bytes("")
-_bytes_newline = _as_bytes("\n")
+__docformat__ = "restructuredtext en"
+
+# For Python 2 can just use: _bgzf_magic = '\x1f\x8b\x08\x04'
+# but need to use bytes on Python 3
+_bgzf_magic = b"\x1f\x8b\x08\x04"
+_bgzf_header = b"\x1f\x8b\x08\x04\x00\x00\x00\x00\x00\xff\x06\x00\x42\x43\x02\x00"
+_bgzf_eof = b"\x1f\x8b\x08\x04\x00\x00\x00\x00\x00\xff\x06\x00BC\x02\x00\x1b\x00\x03\x00\x00\x00\x00\x00\x00\x00\x00\x00"
+_bytes_BC = b"BC"
 
 
 def open(filename, mode="rb"):
@@ -255,16 +260,16 @@ def make_virtual_offset(block_start_offset, within_block_offset):
     The BAM indexing scheme records read positions using a 64 bit
     'virtual offset', comprising in C terms:
 
-    block_start_offset<<16 | within_block_offset
+    block_start_offset << 16 | within_block_offset
 
     Here block_start_offset is the file offset of the BGZF block
     start (unsigned integer using up to 64-16 = 48 bits), and
     within_block_offset within the (decompressed) block (unsigned
     16 bit integer).
 
-    >>> make_virtual_offset(0,0)
+    >>> make_virtual_offset(0, 0)
     0
-    >>> make_virtual_offset(0,1)
+    >>> make_virtual_offset(0, 1)
     1
     >>> make_virtual_offset(0, 2**16 - 1)
     65535
@@ -273,21 +278,21 @@ def make_virtual_offset(block_start_offset, within_block_offset):
     ...
     ValueError: Require 0 <= within_block_offset < 2**16, got 65536
 
-    >>> 65536 == make_virtual_offset(1,0)
+    >>> 65536 == make_virtual_offset(1, 0)
     True
-    >>> 65537 == make_virtual_offset(1,1)
+    >>> 65537 == make_virtual_offset(1, 1)
     True
     >>> 131071 == make_virtual_offset(1, 2**16 - 1)
     True
 
-    >>> 6553600000 == make_virtual_offset(100000,0)
+    >>> 6553600000 == make_virtual_offset(100000, 0)
     True
-    >>> 6553600001 == make_virtual_offset(100000,1)
+    >>> 6553600001 == make_virtual_offset(100000, 1)
     True
-    >>> 6553600010 == make_virtual_offset(100000,10)
+    >>> 6553600010 == make_virtual_offset(100000, 10)
     True
 
-    >>> make_virtual_offset(2**48,0)
+    >>> make_virtual_offset(2**48, 0)
     Traceback (most recent call last):
     ...
     ValueError: Require 0 <= block_start_offset < 2**48, got 281474976710656
@@ -297,7 +302,7 @@ def make_virtual_offset(block_start_offset, within_block_offset):
         raise ValueError("Require 0 <= within_block_offset < 2**16, got %i" % within_block_offset)
     if block_start_offset < 0 or block_start_offset >= 281474976710656:
         raise ValueError("Require 0 <= block_start_offset < 2**48, got %i" % block_start_offset)
-    return (block_start_offset<<16) | within_block_offset
+    return (block_start_offset << 16) | within_block_offset
 
 
 def split_virtual_offset(virtual_offset):
@@ -309,22 +314,31 @@ def split_virtual_offset(virtual_offset):
     True
 
     """
-    start = virtual_offset>>16
-    return start, virtual_offset ^ (start<<16)
+    start = virtual_offset >>16
+    return start, virtual_offset ^ (start << 16)
 
 
 def BgzfBlocks(handle):
     """Low level debugging function to inspect BGZF blocks.
 
+    Expects a BGZF compressed file opened in binary read mode using
+    the builtin open function. Do not use a handle from this bgzf
+    module or the gzip module's open function which will decompress
+    the file.
+
     Returns the block start offset (see virtual offsets), the block
     length (add these for the start of the next block), and the
     decompressed length of the blocks contents (limited to 65536 in
-    BGZF).
+    BGZF), as an iterator - one tuple per BGZF block.
 
-    >>> from __builtin__ import open
+    >>> try:
+    ...     from __builtin__ import open # Python 2
+    ... except ImportError:
+    ...     from builtins import open # Python 3
+    ...
     >>> handle = open("SamBam/ex1.bam", "rb")
     >>> for values in BgzfBlocks(handle):
-    ...     print "Raw start %i, raw length %i; data start %i, data length %i" % values
+    ...     print("Raw start %i, raw length %i; data start %i, data length %i" % values)
     Raw start 0, raw length 18239; data start 0, data length 65536
     Raw start 18239, raw length 18223; data start 65536, data length 65536
     Raw start 36462, raw length 18017; data start 131072, data length 65536
@@ -346,7 +360,7 @@ def BgzfBlocks(handle):
 
     >>> handle = open("SamBam/ex1_refresh.bam", "rb")
     >>> for values in BgzfBlocks(handle):
-    ...     print "Raw start %i, raw length %i; data start %i, data length %i" % values
+    ...     print("Raw start %i, raw length %i; data start %i, data length %i" % values)
     Raw start 0, raw length 53; data start 0, data length 38
     Raw start 53, raw length 18195; data start 38, data length 65434
     Raw start 18248, raw length 18190; data start 65472, data length 65409
@@ -365,7 +379,7 @@ def BgzfBlocks(handle):
 
     >>> handle = open("SamBam/ex1_header.bam", "rb")
     >>> for values in BgzfBlocks(handle):
-    ...     print "Raw start %i, raw length %i; data start %i, data length %i" % values
+    ...     print("Raw start %i, raw length %i; data start %i, data length %i" % values)
     Raw start 0, raw length 104; data start 0, data length 103
     Raw start 104, raw length 18195; data start 103, data length 65434
     Raw start 18299, raw length 18190; data start 65537, data length 65409
@@ -381,7 +395,7 @@ def BgzfBlocks(handle):
     data_start = 0
     while True:
         start_offset = handle.tell()
-        #This may raise StopIteration which is perfect here
+        # This may raise StopIteration which is perfect here
         block_length, data = _load_bgzf_block(handle)
         data_len = len(data)
         yield start_offset, block_length, data_start, data_len
@@ -392,7 +406,7 @@ def _load_bgzf_block(handle, text_mode=False):
     """Internal function to load the next BGZF function (PRIVATE)."""
     magic = handle.read(4)
     if not magic:
-        #End of file
+        # End of file
         raise StopIteration
     if magic != _bgzf_magic:
         raise ValueError(r"A BGZF (e.g. a BAM file) block should start with "
@@ -414,7 +428,7 @@ def _load_bgzf_block(handle, text_mode=False):
             block_size = struct.unpack("<H", subfield_data)[0] + 1  # uint16_t
     assert x_len == extra_len, (x_len, extra_len)
     assert block_size is not None, "Missing BC, this isn't a BGZF file!"
-    #Now comes the compressed data, CRC, and length of uncompressed data.
+    # Now comes the compressed data, CRC, and length of uncompressed data.
     deflate_size = block_size - 1 - extra_len - 19
     d = zlib.decompressobj(-15)  # Negative window size means no headers
     data = d.decompress(handle.read(deflate_size)) + d.flush()
@@ -422,7 +436,7 @@ def _load_bgzf_block(handle, text_mode=False):
     expected_size = struct.unpack("<I", handle.read(4))[0]
     assert expected_size == len(data), \
            "Decompressed to %i, not %i" % (len(data), expected_size)
-    #Should cope with a mix of Python platforms...
+    # Should cope with a mix of Python platforms...
     crc = zlib.crc32(data)
     if crc < 0:
         crc = struct.pack("<i", crc)
@@ -442,10 +456,14 @@ class BgzfReader(object):
     Let's use the BgzfBlocks function to have a peak at the BGZF blocks
     in an example BAM file,
 
-    >>> from __builtin__ import open
+    >>> try:
+    ...     from __builtin__ import open # Python 2
+    ... except ImportError:
+    ...     from builtins import open # Python 3
+    ...
     >>> handle = open("SamBam/ex1.bam", "rb")
     >>> for values in BgzfBlocks(handle):
-    ...     print "Raw start %i, raw length %i; data start %i, data length %i" % values
+    ...     print("Raw start %i, raw length %i; data start %i, data length %i" % values)
     Raw start 0, raw length 18239; data start 0, data length 65536
     Raw start 18239, raw length 18223; data start 65536, data length 65536
     Raw start 36462, raw length 18017; data start 131072, data length 65536
@@ -502,13 +520,13 @@ class BgzfReader(object):
     """
 
     def __init__(self, filename=None, mode="r", fileobj=None, max_cache=100):
-        #TODO - Assuming we can seek, check for 28 bytes EOF empty block
-        #and if missing warn about possible truncation (as in samtools)?
+        # TODO - Assuming we can seek, check for 28 bytes EOF empty block
+        # and if missing warn about possible truncation (as in samtools)?
         if max_cache < 1:
             raise ValueError("Use max_cache with a minimum of 1")
-        #Must open the BGZF file in binary mode, but we may want to
-        #treat the contents as either text or binary (unicode or
-        #bytes under Python 3)
+        # Must open the BGZF file in binary mode, but we may want to
+        # treat the contents as either text or binary (unicode or
+        # bytes under Python 3)
         if fileobj:
             assert filename is None
             handle = fileobj
@@ -517,12 +535,12 @@ class BgzfReader(object):
             if "w" in mode.lower() \
             or "a" in mode.lower():
                 raise ValueError("Must use read mode (default), not write or append mode")
-            handle = __builtin__.open(filename, "rb")
+            handle = _open(filename, "rb")
         self._text = "b" not in mode.lower()
         if self._text:
             self._newline = "\n"
         else:
-            self._newline = _bytes_newline
+            self._newline = b"\n"
         self._handle = handle
         self.max_cache = max_cache
         self._buffers = {}
@@ -532,24 +550,24 @@ class BgzfReader(object):
 
     def _load_block(self, start_offset=None):
         if start_offset is None:
-            #If the file is being read sequentially, then _handle.tell()
-            #should be pointing at the start of the next block.
-            #However, if seek has been used, we can't assume that.
+            # If the file is being read sequentially, then _handle.tell()
+            # should be pointing at the start of the next block.
+            # However, if seek has been used, we can't assume that.
             start_offset = self._block_start_offset + self._block_raw_length
         if start_offset == self._block_start_offset:
             self._within_block_offset = 0
             return
         elif start_offset in self._buffers:
-            #Already in cache
+            # Already in cache
             self._buffer, self._block_raw_length = self._buffers[start_offset]
             self._within_block_offset = 0
             self._block_start_offset = start_offset
             return
-        #Must hit the disk... first check cache limits,
+        # Must hit the disk... first check cache limits,
         while len(self._buffers) >= self.max_cache:
-            #TODO - Implemente LRU cache removal?
+            # TODO - Implemente LRU cache removal?
             self._buffers.popitem()
-        #Now load the block
+        # Now load the block
         handle = self._handle
         if start_offset is not None:
             handle.seek(start_offset)
@@ -557,41 +575,41 @@ class BgzfReader(object):
         try:
             block_size, self._buffer = _load_bgzf_block(handle, self._text)
         except StopIteration:
-            #EOF
+            # EOF
             block_size = 0
             if self._text:
                 self._buffer = ""
             else:
-                self._buffer = _empty_bytes_string
+                self._buffer = b""
         self._within_block_offset = 0
         self._block_raw_length = block_size
-        #Finally save the block in our cache,
+        # Finally save the block in our cache,
         self._buffers[self._block_start_offset] = self._buffer, block_size
 
     def tell(self):
         """Returns a 64-bit unsigned BGZF virtual offset."""
         if 0 < self._within_block_offset == len(self._buffer):
-            #Special case where we're right at the end of a (non empty) block.
-            #For non-maximal blocks could give two possible virtual offsets,
-            #but for a maximal block can't use 65536 as the within block
-            #offset. Therefore for consistency, use the next block and a
-            #within block offset of zero.
+            # Special case where we're right at the end of a (non empty) block.
+            # For non-maximal blocks could give two possible virtual offsets,
+            # but for a maximal block can't use 65536 as the within block
+            # offset. Therefore for consistency, use the next block and a
+            # within block offset of zero.
             return (self._block_start_offset + self._block_raw_length) << 16
         else:
-            #return make_virtual_offset(self._block_start_offset,
+            # return make_virtual_offset(self._block_start_offset,
             #                           self._within_block_offset)
-            #TODO - Include bounds checking as in make_virtual_offset?
-            return (self._block_start_offset<<16) | self._within_block_offset
+            # TODO - Include bounds checking as in make_virtual_offset?
+            return (self._block_start_offset << 16) | self._within_block_offset
 
     def seek(self, virtual_offset):
         """Seek to a 64-bit unsigned BGZF virtual offset."""
-        #Do this inline to avoid a function call,
-        #start_offset, within_block = split_virtual_offset(virtual_offset)
-        start_offset = virtual_offset>>16
-        within_block = virtual_offset ^ (start_offset<<16)
+        # Do this inline to avoid a function call,
+        # start_offset, within_block = split_virtual_offset(virtual_offset)
+        start_offset = virtual_offset >> 16
+        within_block = virtual_offset ^ (start_offset << 16)
         if start_offset != self._block_start_offset:
-            #Don't need to load the block if already there
-            #(this avoids a function call since _load_block would do nothing)
+            # Don't need to load the block if already there
+            # (this avoids a function call since _load_block would do nothing)
             self._load_block(start_offset)
             assert start_offset == self._block_start_offset
         if within_block > len(self._buffer) \
@@ -599,7 +617,7 @@ class BgzfReader(object):
             raise ValueError("Within offset %i but block size only %i"
                              % (within_block, len(self._buffer)))
         self._within_block_offset = within_block
-        #assert virtual_offset == self.tell(), \
+        # assert virtual_offset == self.tell(), \
         #    "Did seek to %i (%i, %i), but tell says %i (%i, %i)" \
         #    % (virtual_offset, start_offset, within_block,
         #       self.tell(), self._block_start_offset, self._within_block_offset)
@@ -612,10 +630,10 @@ class BgzfReader(object):
             if self._text:
                 return ""
             else:
-                return _empty_bytes_string
+                return b""
         elif self._within_block_offset + size <= len(self._buffer):
-            #This may leave us right at the end of a block
-            #(lazy loading, don't load the next block unless we have too)
+            # This may leave us right at the end of a block
+            # (lazy loading, don't load the next block unless we have too)
             data = self._buffer[self._within_block_offset:self._within_block_offset + size]
             self._within_block_offset += size
             assert data  # Must be at least 1 byte
@@ -624,49 +642,54 @@ class BgzfReader(object):
             data = self._buffer[self._within_block_offset:]
             size -= len(data)
             self._load_block()  # will reset offsets
-            #TODO - Test with corner case of an empty block followed by
-            #a non-empty block
+            # TODO - Test with corner case of an empty block followed by
+            # a non-empty block
             if not self._buffer:
                 return data  # EOF
             elif size:
-                #TODO - Avoid recursion
+                # TODO - Avoid recursion
                 return data + self.read(size)
             else:
-                #Only needed the end of the last block
+                # Only needed the end of the last block
                 return data
 
     def readline(self):
         i = self._buffer.find(self._newline, self._within_block_offset)
-        #Three cases to consider,
+        # Three cases to consider,
         if i==-1:
-            #No newline, need to read in more data
+            # No newline, need to read in more data
             data = self._buffer[self._within_block_offset:]
             self._load_block()  # will reset offsets
             if not self._buffer:
                 return data  # EOF
             else:
-                #TODO - Avoid recursion
+                # TODO - Avoid recursion
                 return data + self.readline()
         elif i + 1 == len(self._buffer):
-            #Found new line, but right at end of block (SPECIAL)
+            # Found new line, but right at end of block (SPECIAL)
             data = self._buffer[self._within_block_offset:]
-            #Must now load the next block to ensure tell() works
+            # Must now load the next block to ensure tell() works
             self._load_block()  # will reset offsets
             assert data
             return data
         else:
-            #Found new line, not at end of block (easy case, no IO)
-            data = self._buffer[self._within_block_offset:i+1]
+            # Found new line, not at end of block (easy case, no IO)
+            data = self._buffer[self._within_block_offset:i + 1]
             self._within_block_offset = i + 1
-            #assert data.endswith(self._newline)
+            # assert data.endswith(self._newline)
             return data
 
-    def next(self):
+    def __next__(self):
         line = self.readline()
         if not line:
             raise StopIteration
         return line
 
+    if sys.version_info[0] < 3:
+        def next(self):
+            """Python 2 style alias for Python 3 style __next__ method."""
+            return self.__next__()
+
     def __iter__(self):
         return self
 
@@ -685,6 +708,12 @@ class BgzfReader(object):
     def fileno(self):
         return self._handle.fileno()
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, type, value, traceback):
+        self.close()
+
 
 class BgzfWriter(object):
 
@@ -697,19 +726,19 @@ class BgzfWriter(object):
             and "a" not in mode.lower():
                 raise ValueError("Must use write or append mode, not %r" % mode)
             if "a" in mode.lower():
-                handle = __builtin__.open(filename, "ab")
+                handle = _open(filename, "ab")
             else:
-                handle = __builtin__.open(filename, "wb")
+                handle = _open(filename, "wb")
         self._text = "b" not in mode.lower()
         self._handle = handle
-        self._buffer = _empty_bytes_string
+        self._buffer = b""
         self.compresslevel = compresslevel
 
     def _write_block(self, block):
-        #print "Saving %i bytes" % len(block)
+        # print("Saving %i bytes" % len(block))
         start_offset = self._handle.tell()
         assert len(block) <= 65536
-        #Giving a negative window bits means no gzip/zlib headers, -15 used in samtools
+        # Giving a negative window bits means no gzip/zlib headers, -15 used in samtools
         c = zlib.compressobj(self.compresslevel,
                              zlib.DEFLATED,
                              -15,
@@ -719,36 +748,36 @@ class BgzfWriter(object):
         del c
         assert len(compressed) < 65536, "TODO - Didn't compress enough, try less data in this block"
         crc = zlib.crc32(block)
-        #Should cope with a mix of Python platforms...
+        # Should cope with a mix of Python platforms...
         if crc < 0:
             crc = struct.pack("<i", crc)
         else:
             crc = struct.pack("<I", crc)
-        bsize = struct.pack("<H", len(compressed)+25)  # includes -1
-        crc = struct.pack("<I", zlib.crc32(block) & 0xffffffffL)
+        bsize = struct.pack("<H", len(compressed) + 25)  # includes -1
+        crc = struct.pack("<I", zlib.crc32(block) & 0xffffffff)
         uncompressed_length = struct.pack("<I", len(block))
-        #Fixed 16 bytes,
+        # Fixed 16 bytes,
         # gzip magic bytes (4) mod time (4),
         # gzip flag (1), os (1), extra length which is six (2),
         # sub field which is BC (2), sub field length of two (2),
-        #Variable data,
-        #2 bytes: block length as BC sub field (2)
-        #X bytes: the data
-        #8 bytes: crc (4), uncompressed data length (4)
+        # Variable data,
+        # 2 bytes: block length as BC sub field (2)
+        # X bytes: the data
+        # 8 bytes: crc (4), uncompressed data length (4)
         data = _bgzf_header + bsize + compressed + crc + uncompressed_length
         self._handle.write(data)
 
     def write(self, data):
-        #TODO - Check bytes vs unicode
+        # TODO - Check bytes vs unicode
         data = _as_bytes(data)
-        #block_size = 2**16 = 65536
+        # block_size = 2**16 = 65536
         data_len = len(data)
         if len(self._buffer) + data_len < 65536:
-            #print "Cached %r" % data
+            # print("Cached %r" % data)
             self._buffer += data
             return
         else:
-            #print "Got %r, writing out some data..." % data
+            # print("Got %r, writing out some data..." % data)
             self._buffer += data
             while len(self._buffer) >= 65536:
                 self._write_block(self._buffer[:65536])
@@ -759,16 +788,16 @@ class BgzfWriter(object):
             self._write_block(self._buffer[:65535])
             self._buffer = self._buffer[65535:]
         self._write_block(self._buffer)
-        self._buffer = _empty_bytes_string
+        self._buffer = b""
         self._handle.flush()
 
     def close(self):
         """Flush data, write 28 bytes empty BGZF EOF marker, and close the BGZF file."""
         if self._buffer:
             self.flush()
-        #samtools will look for a magic EOF marker, just a 28 byte empty BGZF block,
-        #and if it is missing warns the BAM file may be truncated. In addition to
-        #samtools writing this block, so too does bgzip - so we should too.
+        # samtools will look for a magic EOF marker, just a 28 byte empty BGZF block,
+        # and if it is missing warns the BAM file may be truncated. In addition to
+        # samtools writing this block, so too does bgzip - so we should too.
         self._handle.write(_bgzf_eof)
         self._handle.flush()
         self._handle.close()
@@ -778,7 +807,7 @@ class BgzfWriter(object):
         return make_virtual_offset(self._handle.tell(), len(self._buffer))
 
     def seekable(self):
-        #Not seekable, but we do support tell...
+        # Not seekable, but we do support tell...
         return False
 
     def isatty(self):
@@ -787,23 +816,29 @@ class BgzfWriter(object):
     def fileno(self):
         return self._handle.fileno()
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, type, value, traceback):
+        self.close()
+
 
 if __name__ == "__main__":
     import sys
     if len(sys.argv) > 1:
-        print "Call this with no arguments and pipe uncompressed data in on stdin"
-        print "and it will produce BGZF compressed data on stdout. e.g."
-        print
-        print "./bgzf.py < example.fastq > example.fastq.bgz"
-        print
-        print "The extension convention of *.bgz is to distinugish these from *.gz"
-        print "used for standard gzipped files without the block structure of BGZF."
-        print "You can use the standard gunzip command to decompress BGZF files,"
-        print "if it complains about the extension try something like this:"
-        print
-        print "cat example.fastq.bgz | gunzip > example.fastq"
-        print
-        print "See also the tool bgzip that comes with samtools"
+        print("Call this with no arguments and pipe uncompressed data in on stdin")
+        print("and it will produce BGZF compressed data on stdout. e.g.")
+        print("")
+        print("./bgzf.py < example.fastq > example.fastq.bgz")
+        print("")
+        print("The extension convention of *.bgz is to distinugish these from *.gz")
+        print("used for standard gzipped files without the block structure of BGZF.")
+        print("You can use the standard gunzip command to decompress BGZF files,")
+        print("if it complains about the extension try something like this:")
+        print("")
+        print("cat example.fastq.bgz | gunzip > example.fastq")
+        print("")
+        print("See also the tool bgzip that comes with samtools")
         sys.exit(0)
 
     sys.stderr.write("Producing BGZF output from stdin...\n")
@@ -813,6 +848,6 @@ if __name__ == "__main__":
         w.write(data)
         if not data:
             break
-    #Doing close with write an empty BGZF block as EOF marker:
+    # Doing close with write an empty BGZF block as EOF marker:
     w.close()
     sys.stderr.write("BGZF data produced\n")
