@@ -13,8 +13,13 @@ and MAST programs, as well as files in the TRANSFAC format.
 Bio.motifs is replacing the older and now obsolete Bio.Motif module.
 """
 
+from __future__ import print_function
+
+from Bio._py3k import range
+
 import math
 
+__docformat__ = "restructuredtext en"
 
 def create(instances, alphabet=None):
     instances = Instances(instances, alphabet)
@@ -25,13 +30,15 @@ def parse(handle, format):
     """Parses an output file of motif finding programs.
 
     Currently supported formats (case is ignored):
-     - AlignAce:      AlignAce output file format
-     - MEME:          MEME output file motif
-     - MAST:          MAST output file motif
-     - TRANSFAC:      TRANSFAC database file format
-     - pfm:           JASPAR-style position-frequency matrix
-     - jaspar:        JASPAR-style multiple PFM format
-     - sites:         JASPAR-style sites file
+
+        - AlignAce:      AlignAce output file format
+        - MEME:          MEME output file motif
+        - MAST:          MAST output file motif
+        - TRANSFAC:      TRANSFAC database file format
+        - pfm:           JASPAR-style position-frequency matrix
+        - jaspar:        JASPAR-style multiple PFM format
+        - sites:         JASPAR-style sites file
+     
     As files in the pfm and sites formats contain only a single motif,
     it is easier to use Bio.motifs.read() instead of Bio.motifs.parse()
     for those.
@@ -39,8 +46,10 @@ def parse(handle, format):
     For example:
 
     >>> from Bio import motifs
-    >>> for m in motifs.parse(open("Motif/alignace.out"),"AlignAce"):
-    ...     print m.consensus
+    >>> with open("Motif/alignace.out") as handle:
+    ...     for m in motifs.parse(handle, "AlignAce"):
+    ...         print(m.consensus)
+    ...
     TCTACGATTGAG
     CTGCAGCTAGCTACGAGTGAG
     GTGCTCTAAGCATAGTAGGCG
@@ -91,14 +100,16 @@ def read(handle, format):
     reading a JASPAR-style pfm file:
 
     >>> from Bio import motifs
-    >>> m = motifs.read(open("motifs/SRF.pfm"), "pfm")
+    >>> with open("motifs/SRF.pfm") as handle:
+    ...     m = motifs.read(handle, "pfm")
     >>> m.consensus
     Seq('GCCCATATATGG', IUPACUnambiguousDNA())
 
     Or a single-motif MEME file,
 
     >>> from Bio import motifs
-    >>> m = motifs.read(open("motifs/meme.out"),"meme")
+    >>> with open("motifs/meme.out") as handle:
+    ...     m = motifs.read(handle, "meme")
     >>> m.consensus
     Seq('CTCAATCGTA', IUPACUnambiguousDNA())
 
@@ -106,7 +117,8 @@ def read(handle, format):
     an exception is raised:
 
     >>> from Bio import motifs
-    >>> motif = motifs.read(open("motifs/alignace.out"),"AlignAce")
+    >>> with open("motifs/alignace.out") as handle:
+    ...     motif = motifs.read(handle, "AlignAce")
     Traceback (most recent call last):
         ...
     ValueError: More than one motif found in handle
@@ -116,7 +128,8 @@ def read(handle, format):
     shown in the example above).  Instead use:
 
     >>> from Bio import motifs
-    >>> record = motifs.parse(open("motifs/alignace.out"),"alignace")
+    >>> with open("motifs/alignace.out") as handle:
+    ...     record = motifs.parse(handle, "alignace")
     >>> motif = record[0]
     >>> motif.consensus
     Seq('TCTACGATTGAG', IUPACUnambiguousDNA())
@@ -187,11 +200,12 @@ class Instances(list):
         """
         a generator function, returning found positions of motif instances in a given sequence
         """
-        for pos in xrange(0,len(sequence)-self.length+1):
+        for pos in range(0, len(sequence)-self.length+1):
             for instance in self:
                 if str(instance) == str(sequence[pos:pos+self.length]):
-                    yield(pos,instance)
+                    yield(pos, instance)
                     break # no other instance will fit (we don't want to return multiple hits)
+
     def reverse_complement(self):
         instances = Instances(alphabet=self.alphabet)
         instances.length = self.length
@@ -206,7 +220,7 @@ class Motif(object):
     A class representing sequence motifs.
     """
     def __init__(self, alphabet=None, instances=None, counts=None):
-        import matrix
+        from . import matrix
         from Bio.Alphabet import IUPAC
         self.name=""
         if counts is not None and instances is not None:
@@ -309,7 +323,7 @@ class Motif(object):
     def pssm(self):
         return self.pwm.log_odds(self._background)
 
-    def __str__(self,masked=False):
+    def __str__(self, masked=False):
         """ string representation of a motif.
         """
         text = ""
@@ -317,7 +331,7 @@ class Motif(object):
             text += str(self.instances)
 
         if masked:
-            for i in xrange(self.length):
+            for i in range(self.length):
                 if self.__mask[i]:
                     text += "*"
                 else:
@@ -375,13 +389,13 @@ Nucleic Acids Research 15(4): 1353-1361. (1987).
 The same rules are used by TRANSFAC."""
         return self.counts.degenerate_consensus
 
-    def weblogo(self,fname,format="PNG",version="2.8.2", **kwds):
+    def weblogo(self, fname, format="PNG", version="2.8.2", **kwds):
         """
         uses the Berkeley weblogo service to download and save a weblogo of
         itself
 
         requires an internet connection.
-        The parameters from **kwds are passed directly to the weblogo server.
+        The parameters from ``**kwds`` are passed directly to the weblogo server.
 
         Currently, this method uses WebLogo version 3.3.
         These are the arguments and their default values passed to
@@ -423,34 +437,34 @@ The same rules are used by TRANSFAC."""
             'color4': '',
 
         """
-        import urllib
-        import urllib2
+        from Bio._py3k import urlopen, urlencode, Request
+
         frequencies = self.format('transfac')
         url = 'http://weblogo.threeplusone.com/create.cgi'
-        values = {'sequences' : frequencies,
-                  'format' : format.lower(),
-                  'stack_width' : 'medium',
-                  'stack_per_line' : '40',
-                  'alphabet' : 'alphabet_dna',
-                  'ignore_lower_case' : True,
-                  'unit_name' : "bits",
-                  'first_index' : '1',
-                  'logo_start' : '1',
+        values = {'sequences': frequencies,
+                  'format': format.lower(),
+                  'stack_width': 'medium',
+                  'stack_per_line': '40',
+                  'alphabet': 'alphabet_dna',
+                  'ignore_lower_case': True,
+                  'unit_name': "bits",
+                  'first_index': '1',
+                  'logo_start': '1',
                   'logo_end': str(self.length),
-                  'composition' : "comp_auto",
-                  'percentCG' : '',
-                  'scale_width' : True,
-                  'show_errorbars' : True,
-                  'logo_title' : '',
-                  'logo_label' : '',
+                  'composition': "comp_auto",
+                  'percentCG': '',
+                  'scale_width': True,
+                  'show_errorbars': True,
+                  'logo_title': '',
+                  'logo_label': '',
                   'show_xaxis': True,
                   'xaxis_label': '',
                   'show_yaxis': True,
                   'yaxis_label': '',
                   'yaxis_scale': 'auto',
-                  'yaxis_tic_interval' : '1.0',
-                  'show_ends' : True,
-                  'show_fineprint' : True,
+                  'yaxis_tic_interval': '1.0',
+                  'show_ends': True,
+                  'show_fineprint': True,
                   'color_scheme': 'color_auto',
                   'symbols0': '',
                   'symbols1': '',
@@ -463,20 +477,18 @@ The same rules are used by TRANSFAC."""
                   'color3': '',
                   'color4': '',
                   }
-        for k,v in kwds.iteritems():
-            if type(values[k])==bool:
+        for k, v in kwds.items():
+            if isinstance(values[k], bool):
                 if not v:
                     v = ""
             values[k]=str(v)
 
-        data = urllib.urlencode(values)
-        req = urllib2.Request(url, data)
-        response = urllib2.urlopen(req)
-        f = open(fname,"w")
-        im = response.read()
-
-        f.write(im)
-        f.close()
+        data = urlencode(values)
+        req = Request(url, data)
+        response = urlopen(req)
+        with open(fname, "w") as f:
+            im = response.read()
+            f.write(im)
 
     def format(self, format):
         """Returns a string representation of the Motif in a given format
@@ -517,3 +529,8 @@ def write(motifs, format):
         return transfac.write(motifs)
     else:
         raise ValueError("Unknown format type %s" % format)
+
+
+if __name__ == "__main__":
+    from Bio._utils import run_doctest
+    run_doctest(verbose=0)
