@@ -2,21 +2,20 @@
 # This code is part of the Biopython distribution and governed by its
 # license. Please see the LICENSE file that should have been included
 # as part of this package.
-"""
-This modules requires MySQLdb to be installed.
+"""Provides read access to a JASPAR5 formatted database.
 
-Provides read access to a JASPAR5 formatted database.
+This modules requires MySQLdb to be installed.
 
 Example, substitute the your database credentials as
 appropriate:
 
     >>> from Bio.motifs.jaspar.db import JASPAR5
-    >>> 
+    >>>
     >>> JASPAR_DB_HOST = "hostname.example.org"
     >>> JASPAR_DB_NAME = "JASPAR_2013"
     >>> JASPAR_DB_USER = "guest"
     >>> JASPAR_DB_PASS = "guest"
-    >>> 
+    >>>
     >>> DFLT_COLLECTION = 'CORE'
     >>> jdb = JASPAR5(
     ...     host=JASPAR_DB_HOST,
@@ -24,10 +23,10 @@ appropriate:
     ...     user=JASPAR_DB_USER,
     ...     password=JASPAR_DB_PASS
     ... )
-    >>> 
-    >>> 
+    >>>
+    >>>
     >>> ets1 = jdb.fetch_motif_by_id('MA0098')
-    >>> print ets1
+    >>> print(ets1)
     TF name ETS1
     Matrix ID   MA0098.1
     Collection  CORE
@@ -48,7 +47,7 @@ appropriate:
     T:  16.00  23.00  39.00   0.00   1.00  15.00
 
 
-    >>> 
+    >>>
     >>> motifs = jdb.fetch_motifs(
     ...     collection = 'CORE',
     ...     tax_group = ['vertebrates', 'insects'],
@@ -56,12 +55,14 @@ appropriate:
     ...     tf_family = ['Forkhead', 'Ets'],
     ...     min_ic = 12
     ... )
-    >>> 
+    >>>
     >>> for motif in motifs:
     ...     pass # do something with the motif
 
 """
 
+from __future__ import print_function
+
 from Bio import MissingPythonDependencyError
 
 try:
@@ -70,14 +71,16 @@ except:
     raise MissingPythonDependencyError("Install MySQLdb if you want to use "
                                        "Bio.motifs.jaspar.db")
 
-
 from Bio.Alphabet.IUPAC import unambiguous_dna as dna
 
 from Bio.motifs import jaspar, matrix
 from warnings import warn
 
+__docformat__ = "restructuredtext en"
+
 JASPAR_DFLT_COLLECTION = 'CORE'
 
+
 class JASPAR5(object):
     """
     Class representing a JASPAR5 DB. The methods within are loosely based
@@ -111,7 +114,7 @@ class JASPAR5(object):
     def __str__(self):
         """
         Return a string represention of the JASPAR5 DB connection.
-        
+
         """
 
         text = "%s\@%s:%s" % (self.user, self.host, self.name)
@@ -124,22 +127,25 @@ class JASPAR5(object):
         (e.g. 'MA0001.1').
 
         Arguments:
-        id - JASPAR matrix ID. This may be a fully specified ID including the
-             version number (e.g. MA0049.2) or just the base ID (e.g. MA0049).
-             If only a base ID is provided, the latest version is returned.
-        Returns:
-        A Bio.motifs.jaspar.Motif object
 
-        NOTE: The perl TFBS module allows you to specify the type of matrix to
+            - id - JASPAR matrix ID. This may be a fully specified ID including the
+              version number (e.g. MA0049.2) or just the base ID (e.g. MA0049).
+              If only a base ID is provided, the latest version is returned.
+
+        Returns:
+
+            - A Bio.motifs.jaspar.Motif object
+
+        **NOTE:** The perl TFBS module allows you to specify the type of matrix to
         return (PFM, PWM, ICM) but matrices are always stored in JASAPR as
         PFMs so this does not really belong here. Once a PFM is fetched the
         pwm() and pssm() methods can be called to return the normalized and
         log-odds matrices.
 
         """
-         
+
         # separate stable ID and version number
-        (base_id, version) = jaspar.split_jaspar_id(id) 
+        (base_id, version) = jaspar.split_jaspar_id(id)
         if not version:
             # if ID contains no version portion, fetch latest version by default
             version = self._fetch_latest_version(base_id)
@@ -155,12 +161,12 @@ class JASPAR5(object):
     def fetch_motifs_by_name(self, name):
         """
         Fetch a list of JASPAR motifs from a JASPAR DB by the given TF name(s).
-        
+
         Arguments:
         name - a single name or list of names
         Returns:
         A list of Bio.motifs.Motif.japar objects
-         
+
         Notes:
         Names are not guaranteed to be unique. There may be more than one
         motif with the same name. Therefore even if name specifies a single
@@ -186,74 +192,76 @@ class JASPAR5(object):
         Fetch a jaspar.Record (list) of motifs based on the provided selection
         criteria.
 
-        Arguments:
-        Except where obvious, all selection criteria arguments may be specified
-        as a single value or a list of values. Motifs must meet ALL the
-        specified selection criteria to be returned with the precedent
-        exceptions noted below.
+        Arguments::
 
-        all         - Takes precedent of all other selection criteria.
-                      Every motif is returned. If 'all_versions' is also
-                      specified, all versions of every motif are returned,
-                      otherwise just the latest version of every motif is
-                      returned.
-        matrix_id   - Takes precedence over all other selection criteria except
-                      'all'.  Only motifs with the given JASPAR matrix ID(s)
-                      are returned. A matrix ID may be specified as just a base
-                      ID or full JASPAR IDs including version number. If only a
-                      base ID is provided for specific motif(s), then just the
-                      latest version of those motif(s) are returned unless
-                      'all_versions' is also specified.
-        collection  - Only motifs from the specified JASPAR collection(s)
-                      are returned. NOTE - if not specified, the collection
-                      defaults to CORE for all other selection criteria except
-                      'all' and 'matrix_id'. To apply the other selection
-                      criteria across all JASPAR collections, explicitly set
-                      collection=None.
-        tf_name     - Only motifs with the given name(s) are returned.
-        tf_class    - Only motifs of the given TF class(es) are returned.
-        tf_family   - Only motifs from the given TF families are returned.
-        tax_group   - Only motifs belonging to the given taxonomic supergroups
-                      are returned (e.g. 'vertebrates', 'insects', 'nematodes'
-                      etc.)
-        species     - Only motifs derived from the given species are returned.
-                      Species are specified as taxonomy IDs.
-        data_type   - Only motifs generated with the given data type (e.g.
-                      ('ChIP-seq', 'PBM', 'SELEX' etc.) are returned. NOTE -
-                      must match exactly as stored in the database.
-        pazar_id    - Only motifs with the given PAZAR TF ID are returned.
-        medline     - Only motifs with the given medline (PubmMed IDs) are 
-                      returned.
-        min_ic      - Only motifs whose profile matrices have at least this
-                      information content (specificty) are returned.
-        min_length  - Only motifs whose profiles are of at least this length
-                      are returned.
-        min_sites   - Only motifs compiled from at least these many binding
-                      sites are returned.
-        all_versions- Unless specified, just the latest version of motifs
-                      determined by the other selection criteria are returned
-                      otherwise all versions of the selected motifs are
-                      returned.
+            Except where obvious, all selection criteria arguments may be specified
+            as a single value or a list of values. Motifs must meet ALL the
+            specified selection criteria to be returned with the precedent
+            exceptions noted below.
+
+            all         - Takes precedent of all other selection criteria.
+                          Every motif is returned. If 'all_versions' is also
+                          specified, all versions of every motif are returned,
+                          otherwise just the latest version of every motif is
+                          returned.
+            matrix_id   - Takes precedence over all other selection criteria except
+                          'all'.  Only motifs with the given JASPAR matrix ID(s)
+                          are returned. A matrix ID may be specified as just a base
+                          ID or full JASPAR IDs including version number. If only a
+                          base ID is provided for specific motif(s), then just the
+                          latest version of those motif(s) are returned unless
+                          'all_versions' is also specified.
+            collection  - Only motifs from the specified JASPAR collection(s)
+                          are returned. NOTE - if not specified, the collection
+                          defaults to CORE for all other selection criteria except
+                          'all' and 'matrix_id'. To apply the other selection
+                          criteria across all JASPAR collections, explicitly set
+                          collection=None.
+            tf_name     - Only motifs with the given name(s) are returned.
+            tf_class    - Only motifs of the given TF class(es) are returned.
+            tf_family   - Only motifs from the given TF families are returned.
+            tax_group   - Only motifs belonging to the given taxonomic supergroups
+                          are returned (e.g. 'vertebrates', 'insects', 'nematodes'
+                          etc.)
+            species     - Only motifs derived from the given species are returned.
+                          Species are specified as taxonomy IDs.
+            data_type   - Only motifs generated with the given data type (e.g.
+                          ('ChIP-seq', 'PBM', 'SELEX' etc.) are returned. NOTE -
+                          must match exactly as stored in the database.
+            pazar_id    - Only motifs with the given PAZAR TF ID are returned.
+            medline     - Only motifs with the given medline (PubmMed IDs) are 
+                          returned.
+            min_ic      - Only motifs whose profile matrices have at least this
+                          information content (specificty) are returned.
+            min_length  - Only motifs whose profiles are of at least this length
+                          are returned.
+            min_sites   - Only motifs compiled from at least these many binding
+                          sites are returned.
+            all_versions- Unless specified, just the latest version of motifs
+                          determined by the other selection criteria are returned
+                          otherwise all versions of the selected motifs are
+                          returned.
 
         Returns:
-        A Bio.motifs.jaspar.Record (list) of motifs.
+
+            - A Bio.motifs.jaspar.Record (list) of motifs.
 
         """
 
         # Fetch the internal IDs of the motifs using the criteria provided
         int_ids = self._fetch_internal_id_list(
-            collection = collection,
-            tf_name = tf_name,
-            tf_class = tf_class,
-            tf_family = tf_family,
-            matrix_id = matrix_id,
-            tax_group = tax_group,
-            species = species,
-            pazar_id = pazar_id,
-            data_type = data_type,
-            medline = medline,
-            all = all,
-            all_versions = all_versions
+            collection=collection,
+            tf_name=tf_name,
+            tf_class=tf_class,
+            tf_family=tf_family,
+            matrix_id=matrix_id,
+            tax_group=tax_group,
+            species=species,
+            pazar_id=pazar_id,
+            data_type=data_type,
+            medline=medline,
+            all=all,
+            all_versions=all_versions
         )
 
         record = jaspar.Record()
@@ -287,7 +295,7 @@ class JASPAR5(object):
                 num_sites = sum(
                     [motif.counts[nt][0] for nt in motif.alphabet.letters]
                 )
-                if  num_sites < min_sites:
+                if num_sites < min_sites:
                     continue
 
             record.append(motif)
@@ -325,7 +333,6 @@ class JASPAR5(object):
 
         return int_id
 
-
     def _fetch_motif_by_internal_id(self, int_id):
         # fetch basic motif information
         sql = "select BASE_ID, VERSION, COLLECTION, NAME from MATRIX where id = %d" % int_id
@@ -335,10 +342,10 @@ class JASPAR5(object):
 
         row = cur.fetchone()
 
-        base_id     = row[0]
-        version     = row[1]
-        collection  = row[2]
-        name        = row[3]
+        base_id = row[0]
+        version = row[1]
+        collection = row[2]
+        name = row[3]
 
         matrix_id = "".join([base_id, '.', str(version)])
 
@@ -347,7 +354,7 @@ class JASPAR5(object):
 
         # Create new JASPAR motif
         motif = jaspar.Motif(
-            matrix_id, name, collection = collection, counts = counts
+            matrix_id, name, collection=collection, counts=counts
         )
 
         # fetch species
@@ -376,7 +383,7 @@ class JASPAR5(object):
         rows = cur.fetchall()
         for row in rows:
             attr = row[0]
-            val  = row[1]
+            val = row[1]
             if attr == 'class':
                 motif.tf_class = val
             elif attr == 'family':
@@ -419,7 +426,7 @@ class JASPAR5(object):
             for row in rows:
                 base_counts.append(row[0])
 
-            counts[base] = map(float, base_counts)
+            counts[base] = [float(x) for x in base_counts]
 
         return matrix.GenericPositionMatrix(dna, counts)
 
@@ -470,12 +477,12 @@ class JASPAR5(object):
         if all:
             cur.execute("select ID from MATRIX")
             rows = cur.fetchall()
-            
+
             for row in rows:
                 int_ids.append(row[0])
 
             return int_ids
-                
+
         """
         Special case 2: fetch specific motifs by their JASPAR IDs. This
         has higher priority than any other except the above 'all' case.
@@ -490,7 +497,7 @@ class JASPAR5(object):
             if all_versions:
                 for id in matrix_id:
                     # ignore vesion here, this is a stupidity filter
-                    (base_id, version) = jaspar.split_jaspar_id(id) 
+                    (base_id, version) = jaspar.split_jaspar_id(id)
                     cur.execute(
                         "select ID from MATRIX where BASE_ID = %s", base_id
                     )
@@ -501,7 +508,7 @@ class JASPAR5(object):
             else:
                 # only the lastest version, or the requested version
                 for id in matrix_id:
-                    (base_id, version) = jaspar.split_jaspar_id(id) 
+                    (base_id, version) = jaspar.split_jaspar_id(id)
 
                     if not version:
                         version = self._fetch_latest_version(base_id)
@@ -691,8 +698,8 @@ class JASPAR5(object):
 
         if where_clauses:
             sql = "".join([sql, " where ", " and ".join(where_clauses)])
-        
-        #print "sql = %s" % sql
+
+        # print "sql = %s" % sql
 
         cur.execute(sql)
         rows = cur.fetchall()
