@@ -8,13 +8,13 @@
 
 try:
     import numpy
+    from numpy import linalg  # missing in PyPy's micronumpy
 except ImportError:
     from Bio import MissingExternalDependencyError
     raise MissingExternalDependencyError(
         "Install NumPy if you want to use Bio.LogisticRegression.")
 
 import unittest
-import sys
 from Bio import LogisticRegression
 
 
@@ -54,6 +54,7 @@ ys = [1,
       0,
       0]
 
+
 class TestLogisticRegression(unittest.TestCase):
 
     def test_calculate_model(self):
@@ -65,14 +66,14 @@ class TestLogisticRegression(unittest.TestCase):
 
     def test_classify(self):
         model = LogisticRegression.train(xs, ys)
-        result = LogisticRegression.classify(model, [6,-173.143442352])
+        result = LogisticRegression.classify(model, [6, -173.143442352])
         self.assertEqual(result, 1)
         result = LogisticRegression.classify(model, [309, -271.005880394])
         self.assertEqual(result, 0)
 
     def test_calculate_probability(self):
         model = LogisticRegression.train(xs, ys)
-        q, p = LogisticRegression.calculate(model, [6,-173.143442352])
+        q, p = LogisticRegression.calculate(model, [6, -173.143442352])
         self.assertAlmostEqual(p, 0.993242, places=6)
         self.assertAlmostEqual(q, 0.006758, places=6)
         q, p = LogisticRegression.calculate(model, [309, -271.005880394])
@@ -102,5 +103,5 @@ class TestLogisticRegression(unittest.TestCase):
         self.assertEqual(correct, 15)
 
 if __name__ == "__main__":
-    runner = unittest.TextTestRunner(verbosity = 2)
+    runner = unittest.TextTestRunner(verbosity=2)
     unittest.main(testRunner=runner)
