@@ -3,6 +3,9 @@
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.
 
+from __future__ import print_function
+from Bio._py3k import _universal_read_mode
+
 import os
 import sys
 import subprocess
@@ -16,27 +19,27 @@ from Bio import AlignIO
 
 #################################################################
 
-#Try to avoid problems when the OS is in another language
+# Try to avoid problems when the OS is in another language
 os.environ['LANG'] = 'C'
 
 muscle_exe = None
 if sys.platform=="win32":
     try:
-        #This can vary depending on the Windows language.
+        # This can vary depending on the Windows language.
         prog_files = os.environ["PROGRAMFILES"]
     except KeyError:
         prog_files = r"C:\Program Files"
-    #For Windows, MUSCLE just comes as a zip file which contains the
-    #a Muscle directory with the muscle.exe file plus a readme etc,
-    #which the user could put anywhere.  We'll try a few sensible
-    #locations under Program Files... and then the full path.
+    # For Windows, MUSCLE just comes as a zip file which contains the
+    # a Muscle directory with the muscle.exe file plus a readme etc,
+    # which the user could put anywhere.  We'll try a few sensible
+    # locations under Program Files... and then the full path.
     likely_dirs = ["",  # Current dir
                    prog_files,
-                   os.path.join(prog_files,"Muscle3.6"),
-                   os.path.join(prog_files,"Muscle3.7"),
-                   os.path.join(prog_files,"Muscle3.8"),
-                   os.path.join(prog_files,"Muscle3.9"),
-                   os.path.join(prog_files,"Muscle")] + sys.path
+                   os.path.join(prog_files, "Muscle3.6"),
+                   os.path.join(prog_files, "Muscle3.7"),
+                   os.path.join(prog_files, "Muscle3.8"),
+                   os.path.join(prog_files, "Muscle3.9"),
+                   os.path.join(prog_files, "Muscle")] + sys.path
     for folder in likely_dirs:
         if os.path.isdir(folder):
             if os.path.isfile(os.path.join(folder, "muscle.exe")):
@@ -45,10 +48,10 @@ if sys.platform=="win32":
         if muscle_exe:
             break
 else:
-    import commands
-    output = commands.getoutput("muscle -version")
-    #Since "not found" may be in another language, try and be sure this is
-    #really the MUSCLE tool's output
+    from Bio._py3k import getoutput
+    output = getoutput("muscle -version")
+    # Since "not found" may be in another language, try and be sure this is
+    # really the MUSCLE tool's output
     if "not found" not in output and "MUSCLE" in output \
     and "Edgar" in output:
         muscle_exe = "muscle"
@@ -98,7 +101,7 @@ class MuscleApplication(unittest.TestCase):
         cmdline = MuscleCommandline(muscle_exe)
         cmdline.set_parameter("input", self.infile1)  # "input" is alias for "in"
         cmdline.set_parameter("out", self.outfile2)
-        #Use property:
+        # Use property:
         cmdline.objscore = "sp"
         cmdline.noanchors = True
         self.assertEqual(str(cmdline), _escape_filename(muscle_exe) +
@@ -129,7 +132,7 @@ class MuscleApplication(unittest.TestCase):
 
     def test_Muscle_profile_with_options(self):
         """Profile alignment, and switch and valued options"""
-        #Using some keyword arguments, note -stable isn't supported in v3.8
+        # Using some keyword arguments, note -stable isn't supported in v3.8
         cmdline = MuscleCommandline(muscle_exe, out=self.outfile4,
                                     in1=self.infile2, in2=self.infile3,
                                     profile=True, stable=True,
@@ -170,8 +173,8 @@ class SimpleAlignTest(unittest.TestCase):
         self.assertEqual(str(cmdline).rstrip(), "muscle -in Fasta/f002 -maxiters 2 -stable")
         result, out_handle, err_handle = generic_run(cmdline)
         #NOTE: generic_run has been removed from Biopython
-        print err_handle.read()
-        print out_handle.read()
+        print(err_handle.read())
+        print(out_handle.read())
         align = AlignIO.read(out_handle, "fasta")
         self.assertEqual(len(records),len(align))
         for old, new in zip(records, align):
@@ -183,10 +186,10 @@ class SimpleAlignTest(unittest.TestCase):
         """Simple muscle call using Clustal output with a MUSCLE header"""
         input_file = "Fasta/f002"
         self.assertTrue(os.path.isfile(input_file))
-        records = list(SeqIO.parse(input_file,"fasta"))
-        records.sort(key = lambda rec: rec.id)
-        #Prepare the command... use Clustal output (with a MUSCLE header)
-        cmdline = MuscleCommandline(muscle_exe, input=input_file, clw = True)
+        records = list(SeqIO.parse(input_file, "fasta"))
+        records.sort(key=lambda rec: rec.id)
+        # Prepare the command... use Clustal output (with a MUSCLE header)
+        cmdline = MuscleCommandline(muscle_exe, input=input_file, clw=True)
         self.assertEqual(str(cmdline).rstrip(), _escape_filename(muscle_exe) +
                          " -in Fasta/f002 -clw")
         self.assertEqual(str(eval(repr(cmdline))), str(cmdline))
@@ -195,7 +198,7 @@ class SimpleAlignTest(unittest.TestCase):
                                  stderr=subprocess.PIPE,
                                  universal_newlines=True,
                                  shell=(sys.platform!="win32"))
-        #Didn't use -quiet so there should be progress reports on stderr,
+        # Didn't use -quiet so there should be progress reports on stderr,
         align = AlignIO.read(child.stdout, "clustal")
         align.sort()  # by record.id
         self.assertTrue(child.stderr.read().strip().startswith("MUSCLE"))
@@ -204,21 +207,21 @@ class SimpleAlignTest(unittest.TestCase):
         child.stdout.close()
         child.stderr.close()
         del child
-        self.assertEqual(len(records),len(align))
+        self.assertEqual(len(records), len(align))
         for old, new in zip(records, align):
             self.assertEqual(old.id, new.id)
-            self.assertEqual(str(new.seq).replace("-",""), str(old.seq))
+            self.assertEqual(str(new.seq).replace("-", ""), str(old.seq))
 
     def test_simple_clustal_strict(self):
         """Simple muscle call using strict Clustal output"""
         input_file = "Fasta/f002"
         self.assertTrue(os.path.isfile(input_file))
-        records = list(SeqIO.parse(input_file,"fasta"))
-        records.sort(key = lambda rec: rec.id)
-        #Prepare the command...
+        records = list(SeqIO.parse(input_file, "fasta"))
+        records.sort(key=lambda rec: rec.id)
+        # Prepare the command...
         cmdline = MuscleCommandline(muscle_exe)
         cmdline.set_parameter("in", input_file)
-        #Use clustal output (with a CLUSTAL header)
+        # Use clustal output (with a CLUSTAL header)
         cmdline.set_parameter("clwstrict", True)  # Default None treated as False!
         self.assertEqual(str(cmdline).rstrip(), _escape_filename(muscle_exe) +
                          " -in Fasta/f002 -clwstrict")
@@ -228,14 +231,14 @@ class SimpleAlignTest(unittest.TestCase):
                                  stderr=subprocess.PIPE,
                                  universal_newlines=True,
                                  shell=(sys.platform!="win32"))
-        #Didn't use -quiet so there should be progress reports on stderr,
+        # Didn't use -quiet so there should be progress reports on stderr,
         align = AlignIO.read(child.stdout, "clustal")
         align.sort()
         self.assertTrue(child.stderr.read().strip().startswith("MUSCLE"))
-        self.assertEqual(len(records),len(align))
+        self.assertEqual(len(records), len(align))
         for old, new in zip(records, align):
             self.assertEqual(old.id, new.id)
-            self.assertEqual(str(new.seq).replace("-",""), str(old.seq))
+            self.assertEqual(str(new.seq).replace("-", ""), str(old.seq))
         return_code = child.wait()
         self.assertEqual(return_code, 0)
         child.stdout.close()
@@ -244,21 +247,21 @@ class SimpleAlignTest(unittest.TestCase):
 
     def test_long(self):
         """Simple muscle call using long file"""
-        #Create a large input file by converting some of another example file
+        # Create a large input file by converting some of another example file
         temp_large_fasta_file = "temp_cw_prot.fasta"
         records = list(SeqIO.parse("NBRF/Cw_prot.pir", "pir"))[:40]
         SeqIO.write(records, temp_large_fasta_file, "fasta")
-        #Prepare the command...
+        # Prepare the command...
         cmdline = MuscleCommandline(muscle_exe)
         cmdline.set_parameter("in", temp_large_fasta_file)
-        #Use fast options
+        # Use fast options
         cmdline.set_parameter("maxiters", 1)
         cmdline.set_parameter("diags", True)  # Default None treated as False!
-        #Use clustal output
+        # Use clustal output
         cmdline.set_parameter("clwstrict", True)  # Default None treated as False!
-        #Shoudn't need this, but just to make sure it is accepted
+        # Shoudn't need this, but just to make sure it is accepted
         cmdline.set_parameter("maxhours", 0.1)
-        #No progress reports to stderr
+        # No progress reports to stderr
         cmdline.set_parameter("quiet", True)  # Default None treated as False!
         self.assertEqual(str(cmdline).rstrip(), _escape_filename(muscle_exe) +
                          " -in temp_cw_prot.fasta -diags -maxhours 0.1" +
@@ -271,12 +274,12 @@ class SimpleAlignTest(unittest.TestCase):
                                  shell=(sys.platform!="win32"))
         align = AlignIO.read(child.stdout, "clustal")
         align.sort()
-        records.sort(key = lambda rec: rec.id)
+        records.sort(key=lambda rec: rec.id)
         self.assertEqual(len(records), len(align))
         for old, new in zip(records, align):
             self.assertEqual(old.id, new.id)
-            self.assertEqual(str(new.seq).replace("-",""), str(old.seq))
-        #See if quiet worked:
+            self.assertEqual(str(new.seq).replace("-", ""), str(old.seq))
+        # See if quiet worked:
         self.assertEqual("", child.stderr.read().strip())
         return_code = child.wait()
         self.assertEqual(return_code, 0)
@@ -289,8 +292,8 @@ class SimpleAlignTest(unittest.TestCase):
         """Simple alignment using stdin"""
         input_file = "Fasta/f002"
         self.assertTrue(os.path.isfile(input_file))
-        records = list(SeqIO.parse(input_file,"fasta"))
-        #Prepare the command... use Clustal output (with a MUSCLE header)
+        records = list(SeqIO.parse(input_file, "fasta"))
+        # Prepare the command... use Clustal output (with a MUSCLE header)
         cline = MuscleCommandline(muscle_exe, clw=True)
         self.assertEqual(str(cline).rstrip(),
                          _escape_filename(muscle_exe) + " -clw")
@@ -303,14 +306,14 @@ class SimpleAlignTest(unittest.TestCase):
                                  shell=(sys.platform!="win32"))
         SeqIO.write(records, child.stdin, "fasta")
         child.stdin.close()
-        #Alignment will now run...
+        # Alignment will now run...
         align = AlignIO.read(child.stdout, "clustal")
         align.sort()
-        records.sort(key = lambda rec: rec.id)
-        self.assertEqual(len(records),len(align))
+        records.sort(key=lambda rec: rec.id)
+        self.assertEqual(len(records), len(align))
         for old, new in zip(records, align):
             self.assertEqual(old.id, new.id)
-            self.assertEqual(str(new.seq).replace("-",""), str(old.seq))
+            self.assertEqual(str(new.seq).replace("-", ""), str(old.seq))
         self.assertEqual(0, child.wait())
         child.stdout.close()
         child.stderr.close()
@@ -322,12 +325,12 @@ class SimpleAlignTest(unittest.TestCase):
         output_html = "temp_f002.html"
         output_clwstrict = "temp_f002.clw"
         self.assertTrue(os.path.isfile(input_file))
-        records = list(SeqIO.parse(input_file,"fasta"))
-        records.sort(key = lambda rec: rec.id)
-        #Prepare the command... use Clustal output (with a MUSCLE header)
+        records = list(SeqIO.parse(input_file, "fasta"))
+        records.sort(key=lambda rec: rec.id)
+        # Prepare the command... use Clustal output (with a MUSCLE header)
         cmdline = MuscleCommandline(muscle_exe, input=input_file,
-                                    clw=True, htmlout = output_html,
-                                    clwstrictout = output_clwstrict)
+                                    clw=True, htmlout=output_html,
+                                    clwstrictout=output_clwstrict)
         self.assertEqual(str(cmdline).rstrip(), _escape_filename(muscle_exe) +
                          " -in Fasta/f002 -clw -htmlout temp_f002.html" +
                          " -clwstrictout temp_f002.clw")
@@ -337,33 +340,33 @@ class SimpleAlignTest(unittest.TestCase):
                                  stderr=subprocess.PIPE,
                                  universal_newlines=True,
                                  shell=(sys.platform!="win32"))
-        #Clustalw on stdout:
+        # Clustalw on stdout:
         align = AlignIO.read(child.stdout, "clustal")
         align.sort()
-        #Didn't use -quiet so there should be progress reports on stderr,
+        # Didn't use -quiet so there should be progress reports on stderr,
         self.assertTrue(child.stderr.read().strip().startswith("MUSCLE"))
         return_code = child.wait()
         self.assertEqual(return_code, 0)
-        self.assertEqual(len(records),len(align))
+        self.assertEqual(len(records), len(align))
         for old, new in zip(records, align):
             self.assertEqual(old.id, new.id)
         child.stdout.close()
         child.stderr.close()
         del child
-        handle = open(output_html,"rU")
+        handle = open(output_html, _universal_read_mode)
         html = handle.read().strip().upper()
         handle.close()
         self.assertTrue(html.startswith("<HTML"))
         self.assertTrue(html.endswith("</HTML>"))
-        #ClustalW strict:
+        # ClustalW strict:
         align = AlignIO.read(output_clwstrict, "clustal")
         align.sort()
-        self.assertEqual(len(records),len(align))
+        self.assertEqual(len(records), len(align))
         for old, new in zip(records, align):
             self.assertEqual(old.id, new.id)
         os.remove(output_html)
         os.remove(output_clwstrict)
 
 if __name__ == "__main__":
-    runner = unittest.TextTestRunner(verbosity = 2)
+    runner = unittest.TextTestRunner(verbosity=2)
     unittest.main(testRunner=runner)
