@@ -4,23 +4,28 @@
 # as part of this package.
 
 """Unittests for the Seq objects."""
+from __future__ import print_function
+
+import warnings
 import unittest
 import sys
-if sys.version_info[0] == 3:
+if sys.version_info[0] >= 3:
     maketrans = str.maketrans
 else:
     from string import maketrans
 
+from Bio import BiopythonWarning
 from Bio.Alphabet import generic_protein, generic_nucleotide, \
                          generic_dna, generic_rna
+from Bio.Alphabet import _check_type_compatible
 from Bio.Alphabet.IUPAC import protein, extended_protein
 from Bio.Alphabet.IUPAC import unambiguous_dna, ambiguous_dna, ambiguous_rna
 from Bio.Data.IUPACData import ambiguous_dna_values, ambiguous_rna_values
 from Bio.Seq import Seq, UnknownSeq, MutableSeq, translate
 from Bio.Data.CodonTable import TranslationError, CodonTable
 
-#This is just the standard table with less stop codons
-#(replaced with coding for O as an artifical example)
+# This is just the standard table with less stop codons
+# (replaced with coding for O as an artifical example)
 special_table = CodonTable(forward_table={
     'TTT': 'F', 'TTC': 'F', 'TTA': 'L', 'TTG': 'L',
     'TCT': 'S', 'TCC': 'S', 'TCA': 'S', 'TCG': 'S',
@@ -58,13 +63,13 @@ Chilodonella_uncinata_table = CodonTable(forward_table={
     'GCT': 'A', 'GCC': 'A', 'GCA': 'A', 'GCG': 'A',
     'GAT': 'D', 'GAC': 'D', 'GAA': 'E', 'GAG': 'E',
     'GGT': 'G', 'GGC': 'G', 'GGA': 'G', 'GGG': 'G'},
-    start_codons = [ 'ATG'],
-    stop_codons = ['TAA' ])
+    start_codons=['ATG'],
+    stop_codons=['TAA'])
 
 
 class StringMethodTests(unittest.TestCase):
     _examples = [
-        #These are length 9, a multiple of 3 for translation tests:
+        # These are length 9, a multiple of 3 for translation tests:
         Seq("ACGTGGGGT", generic_protein),
         Seq("ACGTGGGGT", generic_nucleotide),
         Seq("ACGTGGGGT", generic_dna),
@@ -99,18 +104,18 @@ class StringMethodTests(unittest.TestCase):
         self.assertTrue(isinstance(method_name, str))
         for example1 in self._examples:
             if not hasattr(example1, method_name):
-                #e.g. MutableSeq does not support find
+                # e.g. MutableSeq does not support find
                 continue
             str1 = str(example1)
 
             for example2 in self._examples:
                 if not hasattr(example2, method_name):
-                    #e.g. MutableSeq does not support find
+                    # e.g. MutableSeq does not support find
                     continue
                 str2 = str(example2)
 
-                i = getattr(example1,method_name)(str2)
-                j = getattr(str1,method_name)(str2)
+                i = getattr(example1, method_name)(str2)
+                j = getattr(str1, method_name)(str2)
                 if pre_comp_function:
                     i = pre_comp_function(i)
                     j = pre_comp_function(j)
@@ -123,8 +128,8 @@ class StringMethodTests(unittest.TestCase):
                                         j))
 
                 try:
-                    i = getattr(example1,method_name)(example2)
-                    j = getattr(str1,method_name)(str2)
+                    i = getattr(example1, method_name)(example2)
+                    j = getattr(str1, method_name)(str2)
                     if pre_comp_function:
                         i = pre_comp_function(i)
                         j = pre_comp_function(j)
@@ -136,13 +141,13 @@ class StringMethodTests(unittest.TestCase):
                                             i,
                                             j))
                 except TypeError:
-                    #TODO - Check the alphabets do clash!
+                    # TODO - Check the alphabets do clash!
                     pass
 
                 if start_end:
                     for start in self._start_end_values:
-                        i = getattr(example1,method_name)(str2, start)
-                        j = getattr(str1,method_name)(str2, start)
+                        i = getattr(example1, method_name)(str2, start)
+                        j = getattr(str1, method_name)(str2, start)
                         if pre_comp_function:
                             i = pre_comp_function(i)
                             j = pre_comp_function(j)
@@ -156,8 +161,8 @@ class StringMethodTests(unittest.TestCase):
                                                 j))
 
                         for end in self._start_end_values:
-                            i = getattr(example1,method_name)(str2, start, end)
-                            j = getattr(str1,method_name)(str2, start, end)
+                            i = getattr(example1, method_name)(str2, start, end)
+                            j = getattr(str1, method_name)(str2, start, end)
                             if pre_comp_function:
                                 i = pre_comp_function(i)
                                 j = pre_comp_function(j)
@@ -186,58 +191,48 @@ class StringMethodTests(unittest.TestCase):
     def test_str_startswith(self):
         """Check matches the python string startswith method."""
         self._test_method("startswith", start_end=True)
+        self.assertTrue("ABCDE".startswith(("ABE", "OBE", "ABC")))
 
-        try:
-            self.assertTrue("ABCDE".startswith(("ABE","OBE","ABC")))
-        except TypeError:
-            #Base string only supports this on Python 2.5+, skip this
-            return
-
-        #Now check with a tuple of sub sequences
+        # Now check with a tuple of sub sequences
         for example1 in self._examples:
             if not hasattr(example1, "startswith"):
-                #e.g. MutableSeq does not support this
+                # e.g. MutableSeq does not support this
                 continue
             subs = tuple([example1[start:start+2] for start
-                          in range(0, len(example1)-2,3)])
+                          in range(0, len(example1)-2, 3)])
             subs_str = tuple([str(s) for s in subs])
 
             self.assertEqual(str(example1).startswith(subs_str),
                              example1.startswith(subs))
             self.assertEqual(str(example1).startswith(subs_str),
                              example1.startswith(subs_str))  # strings!
-            self.assertEqual(str(example1).startswith(subs_str,3),
-                             example1.startswith(subs,3))
-            self.assertEqual(str(example1).startswith(subs_str,2,6),
-                             example1.startswith(subs,2,6))
+            self.assertEqual(str(example1).startswith(subs_str, 3),
+                             example1.startswith(subs, 3))
+            self.assertEqual(str(example1).startswith(subs_str, 2, 6),
+                             example1.startswith(subs, 2, 6))
 
     def test_str_endswith(self):
         """Check matches the python string endswith method."""
         self._test_method("endswith", start_end=True)
+        self.assertTrue("ABCDE".endswith(("ABE", "OBE", "CDE")))
 
-        try:
-            self.assertTrue("ABCDE".endswith(("ABE","OBE","CDE")))
-        except TypeError:
-            #Base string only supports this on Python 2.5+, skip this
-            return
-
-        #Now check with a tuple of sub sequences
+        # Now check with a tuple of sub sequences
         for example1 in self._examples:
             if not hasattr(example1, "endswith"):
-                #e.g. MutableSeq does not support this
+                # e.g. MutableSeq does not support this
                 continue
             subs = tuple([example1[start:start+2] for start
-                          in range(0, len(example1)-2,3)])
+                          in range(0, len(example1)-2, 3)])
             subs_str = tuple([str(s) for s in subs])
 
             self.assertEqual(str(example1).endswith(subs_str),
                              example1.endswith(subs))
             self.assertEqual(str(example1).startswith(subs_str),
                              example1.startswith(subs_str))  # strings!
-            self.assertEqual(str(example1).endswith(subs_str,3),
-                             example1.endswith(subs,3))
-            self.assertEqual(str(example1).endswith(subs_str,2,6),
-                             example1.endswith(subs,2,6))
+            self.assertEqual(str(example1).endswith(subs_str, 3),
+                             example1.endswith(subs, 3))
+            self.assertEqual(str(example1).endswith(subs_str, 2, 6),
+                             example1.endswith(subs, 2, 6))
 
     def test_str_strip(self):
         """Check matches the python string strip method."""
@@ -249,21 +244,24 @@ class StringMethodTests(unittest.TestCase):
 
     def test_str_split(self):
         """Check matches the python string rstrip method."""
-        #Calling (r)split should return a list of Seq-like objects, we'll
-        #just apply str() to each of them so it matches the string method
-        self._test_method("rstrip", pre_comp_function=lambda x : map(str,x))
+        # Calling (r)split should return a list of Seq-like objects, we'll
+        # just apply str() to each of them so it matches the string method
+        self._test_method("rstrip",
+                          pre_comp_function=lambda x: [str(y) for y in x])
 
     def test_str_rsplit(self):
         """Check matches the python string rstrip method."""
-        #Calling (r)split should return a list of Seq-like objects, we'll
-        #just apply str() to each of them so it matches the string method
-        self._test_method("rstrip", pre_comp_function=lambda x : map(str,x))
+        # Calling (r)split should return a list of Seq-like objects, we'll
+        # just apply str() to each of them so it matches the string method
+        self._test_method("rstrip",
+                          pre_comp_function=lambda x: [str(y) for y in x])
 
     def test_str_lsplit(self):
         """Check matches the python string rstrip method."""
-        #Calling (r)split should return a list of Seq-like objects, we'll
-        #just apply str() to each of them so it matches the string method
-        self._test_method("rstrip", pre_comp_function=lambda x : map(str,x))
+        # Calling (r)split should return a list of Seq-like objects, we'll
+        # just apply str() to each of them so it matches the string method
+        self._test_method("rstrip",
+                          pre_comp_function=lambda x: [str(y) for y in x])
 
     def test_str_length(self):
         """Check matches the python string __len__ method."""
@@ -287,6 +285,43 @@ class StringMethodTests(unittest.TestCase):
             str1 = str(example1)
             self.assertEqual(str(example1.lower()), str1.lower())
 
+    def test_str_hash(self):
+        for example1 in self._examples:
+            if isinstance(example1, MutableSeq):
+                continue
+            with warnings.catch_warnings():
+                #Silence change in behaviour warning
+                warnings.simplefilter('ignore', BiopythonWarning)
+                self.assertEqual(hash(str(example1)), hash(example1),
+                                 "Hash mismatch, %r for %r vs %r for %r"
+                                 % (hash(str(example1)), id(example1),
+                                    hash(example1), example1))
+
+    def test_str_comparison(self):
+        for example1 in self._examples:
+            for example2 in self._examples:
+                with warnings.catch_warnings():
+                    #Silence alphabet warning
+                    warnings.simplefilter('ignore', BiopythonWarning)
+                    self.assertEqual(str(example1) == str(example2),
+                                     example1 == example2,
+                                     "Checking %r == %r" % (example1, example2))
+                    self.assertEqual(str(example1) != str(example2),
+                                     example1 != example2,
+                                     "Checking %r != %r" % (example1, example2))
+                    self.assertEqual(str(example1) < str(example2),
+                                     example1 < example2,
+                                     "Checking %r < %r" % (example1, example2))
+                    self.assertEqual(str(example1) <= str(example2),
+                                     example1 <= example2,
+                                     "Checking %r <= %r" % (example1, example2))
+                    self.assertEqual(str(example1) > str(example2),
+                                     example1 > example2,
+                                     "Checking %r > %r" % (example1, example2))
+                    self.assertEqual(str(example1) >= str(example2),
+                                     example1 >= example2,
+                                     "Checking %r >= %r" % (example1, example2))
+
     def test_str_getitem(self):
         """Check slicing and indexing works like a string."""
         for example1 in self._examples:
@@ -298,10 +333,10 @@ class StringMethodTests(unittest.TestCase):
                 self.assertEqual(str(example1[i:]), str1[i:])
                 for j in self._start_end_values:
                     self.assertEqual(str(example1[i:j]), str1[i:j])
-                    for step in range(-3,4):
+                    for step in range(-3, 4):
                         if step == 0:
                             try:
-                                print example1[i:j:step]
+                                print(example1[i:j:step])
                                 self._assert(False)  # Should fail!
                             except ValueError:
                                 pass
@@ -309,12 +344,6 @@ class StringMethodTests(unittest.TestCase):
                             self.assertEqual(str(example1[i:j:step]),
                                              str1[i:j:step])
 
-    def test_tostring(self):
-        """Check str(obj) and obj.tostring() match."""
-        for example1 in self._examples:
-            str1 = str(example1)
-            self.assertEqual(example1.tostring(), str1)
-
     def test_tomutable(self):
         """Check obj.tomutable() method."""
         for example1 in self._examples:
@@ -328,9 +357,9 @@ class StringMethodTests(unittest.TestCase):
     def test_toseq(self):
         """Check obj.toseq() method."""
         for example1 in self._examples:
-            try :
+            try:
                 seq = example1.toseq()
-            except AttributeError :
+            except AttributeError:
                 self.assertTrue(isinstance(example1, Seq))
                 continue
             self.assertTrue(isinstance(seq, Seq))
@@ -343,24 +372,24 @@ class StringMethodTests(unittest.TestCase):
         for example1 in self._examples:
             if isinstance(example1, MutableSeq):
                 continue
-            try :
+            try:
                 comp = example1.complement()
-            except ValueError, e:
+            except ValueError as e:
                 self.assertEqual(str(e), "Proteins do not have complements!")
                 continue
             str1 = str(example1)
-            #This only does the unambiguous cases
+            # This only does the unambiguous cases
             if "U" in str1 or "u" in str1 \
             or example1.alphabet==generic_rna:
-                mapping = maketrans("ACGUacgu","UGCAugca")
+                mapping = maketrans("ACGUacgu", "UGCAugca")
             elif "T" in str1 or "t" in str1 \
             or example1.alphabet==generic_dna \
             or example1.alphabet==generic_nucleotide:
-                mapping = maketrans("ACGTacgt","TGCAtgca")
+                mapping = maketrans("ACGTacgt", "TGCAtgca")
             elif "A" not in str1 and "a" not in str1:
-                mapping = maketrans("CGcg","GCgc")
-            else :
-                #TODO - look at alphabet?
+                mapping = maketrans("CGcg", "GCgc")
+            else:
+                # TODO - look at alphabet?
                 raise ValueError(example1)
             self.assertEqual(str1.translate(mapping), str(comp))
             self.assertEqual(comp.alphabet, example1.alphabet)
@@ -371,24 +400,24 @@ class StringMethodTests(unittest.TestCase):
         for example1 in self._examples:
             if isinstance(example1, MutableSeq):
                 continue
-            try :
+            try:
                 comp = example1.reverse_complement()
-            except ValueError, e:
+            except ValueError as e:
                 self.assertEqual(str(e), "Proteins do not have complements!")
                 continue
             str1 = str(example1)
-            #This only does the unambiguous cases
+            # This only does the unambiguous cases
             if "U" in str1 or "u" in str1 \
             or example1.alphabet==generic_rna:
-                mapping = maketrans("ACGUacgu","UGCAugca")
+                mapping = maketrans("ACGUacgu", "UGCAugca")
             elif "T" in str1 or "t" in str1 \
             or example1.alphabet==generic_dna \
             or example1.alphabet==generic_nucleotide:
-                mapping = maketrans("ACGTacgt","TGCAtgca")
+                mapping = maketrans("ACGTacgt", "TGCAtgca")
             elif "A" not in str1 and "a" not in str1:
-                mapping = maketrans("CGcg","GCgc")
-            else :
-                #TODO - look at alphabet?
+                mapping = maketrans("CGcg", "GCgc")
+            else:
+                # TODO - look at alphabet?
                 continue
             self.assertEqual(str1.translate(mapping)[::-1], str(comp))
             self.assertEqual(comp.alphabet, example1.alphabet)
@@ -399,9 +428,9 @@ class StringMethodTests(unittest.TestCase):
             for example1 in self._examples:
                 if isinstance(example1, MutableSeq):
                     continue
-                try :
+                try:
                     tran = example1.transcribe()
-                except ValueError, e:
+                except ValueError as e:
                     if str(e) == "Proteins cannot be transcribed!":
                         continue
                     if str(e) == "RNA cannot be transcribed!":
@@ -409,9 +438,9 @@ class StringMethodTests(unittest.TestCase):
                     raise e
                 str1 = str(example1)
                 if len(str1) % 3 != 0:
-                    #TODO - Check for or silence the expected warning?
+                    # TODO - Check for or silence the expected warning?
                     continue
-                self.assertEqual(str1.replace("T","U").replace("t","u"), str(tran))
+                self.assertEqual(str1.replace("T", "U").replace("t", "u"), str(tran))
                 self.assertEqual(tran.alphabet, generic_rna)  # based on limited examples
 
     def test_the_back_transcription(self):
@@ -420,16 +449,16 @@ class StringMethodTests(unittest.TestCase):
             for example1 in self._examples:
                 if isinstance(example1, MutableSeq):
                     continue
-                try :
+                try:
                     tran = example1.back_transcribe()
-                except ValueError, e:
+                except ValueError as e:
                     if str(e) == "Proteins cannot be back transcribed!":
                         continue
                     if str(e) == "DNA cannot be back transcribed!":
                         continue
                     raise e
                 str1 = str(example1)
-                self.assertEqual(str1.replace("U","T").replace("u","t"), str(tran))
+                self.assertEqual(str1.replace("U", "T").replace("u", "t"), str(tran))
                 self.assertEqual(tran.alphabet, generic_dna)  # based on limited examples
 
     def test_the_translate(self):
@@ -439,19 +468,19 @@ class StringMethodTests(unittest.TestCase):
                 if isinstance(example1, MutableSeq):
                     continue
                 if len(example1) % 3 != 0:
-                    #TODO - Check for or silence the expected warning?
+                    # TODO - Check for or silence the expected warning?
                     continue
-                try :
+                try:
                     tran = example1.translate()
-                except ValueError, e:
+                except ValueError as e:
                     if str(e) == "Proteins cannot be translated!":
                         continue
                     raise e
-                #This is based on the limited example not having stop codons:
+                # This is based on the limited example not having stop codons:
                 if tran.alphabet not in [extended_protein, protein, generic_protein]:
-                    print tran.alphabet
+                    print(tran.alphabet)
                     self.assertTrue(False)
-                #TODO - check the actual translation, and all the optional args
+                # TODO - check the actual translation, and all the optional args
 
     def test_the_translation_of_stops(self):
         """Check obj.translate() method with stop codons."""
@@ -474,7 +503,7 @@ class StringMethodTests(unittest.TestCase):
             self.assertEqual("", str(nuc.translate(to_stop=True)))
             self.assertEqual("O*ORR", str(nuc.translate(table=special_table)))
             self.assertEqual("*QWRR", str(nuc.translate(table=Chilodonella_uncinata_table)))
-            #These test the Bio.Seq.translate() function - move these?:
+            # These test the Bio.Seq.translate() function - move these?:
             self.assertEqual("*QWRR", translate(str(nuc), table=Chilodonella_uncinata_table))
             self.assertEqual("O*ORR", translate(str(nuc), table=special_table))
             self.assertEqual("", translate(str(nuc), to_stop=True))
@@ -502,16 +531,16 @@ class StringMethodTests(unittest.TestCase):
                         Seq(codon, generic_nucleotide),
                         Seq(codon, generic_dna),
                         Seq(codon, unambiguous_dna)]:
-                try :
-                    print nuc.translate()
+                try:
+                    print(nuc.translate())
                     self.assertTrue(False, "Transating %s should fail" % codon)
-                except TranslationError :
+                except TranslationError:
                     pass
 
     def test_the_translation_of_ambig_codons(self):
         """Check obj.translate() method with ambiguous codons."""
         for letters, ambig_values in [(ambiguous_dna.letters, ambiguous_dna_values),
-                                      (ambiguous_rna.letters, ambiguous_rna_values)] :
+                                      (ambiguous_rna.letters, ambiguous_rna_values)]:
             ambig = set(letters)
             for c1 in ambig:
                 for c2 in ambig:
@@ -535,17 +564,17 @@ class StringMethodTests(unittest.TestCase):
                             self.assertEqual(values, set("LI"))
                         else:
                             self.assertEqual(values, set(t))
-                        #TODO - Use the Bio.Data.IUPACData module for the
-                        #ambiguous protein mappings?
+                        # TODO - Use the Bio.Data.IUPACData module for the
+                        # ambiguous protein mappings?
 
     def test_init_typeerror(self):
         """Check Seq __init__ gives TypeError exceptions."""
-        #Only expect it to take strings and unicode - not Seq objects!
+        # Only expect it to take strings and unicode - not Seq objects!
         self.assertRaises(TypeError, Seq, (1066))
         self.assertRaises(TypeError, Seq, (Seq("ACGT", generic_dna)))
 
-    #TODO - Addition...
+    # TODO - Addition...
 
 if __name__ == "__main__":
-    runner = unittest.TextTestRunner(verbosity = 2)
+    runner = unittest.TextTestRunner(verbosity=2)
     unittest.main(testRunner=runner)
